@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: cu_algorithm.c 13696 2015-06-24 19:06:01Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
@@ -425,15 +424,19 @@ static void test_point_interpolate(void)
 	q.m = 50.0;
 
 	rv = point_interpolate(&p, &q, &r, 1, 1, 'Z', 35.0);
+	CU_ASSERT_EQUAL( rv, LW_SUCCESS );
 	CU_ASSERT_EQUAL( r.x, 15.0);
 
 	rv = point_interpolate(&p, &q, &r, 1, 1, 'M', 41.0);
+	CU_ASSERT_EQUAL( rv, LW_SUCCESS );
 	CU_ASSERT_EQUAL( r.y, 21.0);
 
 	rv = point_interpolate(&p, &q, &r, 1, 1, 'M', 50.0);
+	CU_ASSERT_EQUAL( rv, LW_SUCCESS );
 	CU_ASSERT_EQUAL( r.y, 30.0);
 
 	rv = point_interpolate(&p, &q, &r, 1, 1, 'M', 40.0);
+	CU_ASSERT_EQUAL( rv, LW_SUCCESS );
 	CU_ASSERT_EQUAL( r.y, 20.0);
 
 }
@@ -883,40 +886,76 @@ static void test_geohash_point_as_int(void)
 {
 	unsigned int gh;
 	POINT2D p;
+	unsigned long long rs;
 	
 	p.x = 50; p.y = 35;
 	gh = geohash_point_as_int(&p);
-	CU_ASSERT_EQUAL(gh, (unsigned int)3440103613);
+	rs = 3440103613;
+	CU_ASSERT_EQUAL(gh, rs);
 	p.x = 140; p.y = 45;
 	gh = geohash_point_as_int(&p);
-	CU_ASSERT_EQUAL(gh, (unsigned int)3982480893);
+	rs = 3982480893;
+	CU_ASSERT_EQUAL(gh, rs);
 	p.x = 140; p.y = 55;
 	gh = geohash_point_as_int(&p);
-	CU_ASSERT_EQUAL(gh, (unsigned int)4166944232);	
+	rs = 4166944232;
+	CU_ASSERT_EQUAL(gh, rs);
 }
 
 static void test_lwgeom_simplify(void)
 {
 		LWGEOM *l;
-		LWGEOM *i;
+		LWGEOM *g;
 		char *ewkt;
 
+		/* Simplify but only so far... */
+		g = lwgeom_from_wkt("LINESTRING(0 0, 1 0, 1 1, 0 1, 0 0)", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 10, LW_TRUE);
+		ewkt = lwgeom_to_ewkt(l);
+		CU_ASSERT_STRING_EQUAL(ewkt, "LINESTRING(0 0,0 0)");
+		lwgeom_free(g);
+		lwgeom_free(l);
+		lwfree(ewkt);
+
+		/* Simplify but only so far... */
+		g = lwgeom_from_wkt("POLYGON((0 0, 1 0, 1 1, 0 1, 0 0))", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 10, LW_TRUE);
+		ewkt = lwgeom_to_ewkt(l);
+		CU_ASSERT_STRING_EQUAL(ewkt, "POLYGON((0 0,1 0,1 1,0 0))");
+		lwgeom_free(g);
+		lwgeom_free(l);
+		lwfree(ewkt);
+
+		/* Simplify and collapse */
+		g = lwgeom_from_wkt("LINESTRING(0 0, 1 0, 1 1, 0 1, 0 0)", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 10, LW_FALSE);
+		CU_ASSERT_EQUAL(l, NULL);
+		lwgeom_free(g);
+		lwgeom_free(l);
+
+		/* Simplify and collapse */
+		g = lwgeom_from_wkt("POLYGON((0 0, 1 0, 1 1, 0 1, 0 0))", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 10, LW_FALSE);
+		CU_ASSERT_EQUAL(l, NULL);
+		lwgeom_free(g);
+		lwgeom_free(l);
+		
 		/* Not simplifiable */
-		i = lwgeom_from_wkt("LINESTRING(0 0, 50 1.00001, 100 0)", LW_PARSER_CHECK_NONE);
-		l = lwgeom_simplify(i, 1.0);
+		g = lwgeom_from_wkt("LINESTRING(0 0, 50 1.00001, 100 0)", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 1.0, LW_FALSE);
 		ewkt = lwgeom_to_ewkt(l);
 		CU_ASSERT_STRING_EQUAL(ewkt, "LINESTRING(0 0,50 1.00001,100 0)");
+		lwgeom_free(g);
 		lwgeom_free(l);
-		lwgeom_free(i);
 		lwfree(ewkt);
 
 		/* Simplifiable */
-		i = lwgeom_from_wkt("LINESTRING(0 0,50 0.99999,100 0)", LW_PARSER_CHECK_NONE);
-		l = lwgeom_simplify(i, 1.0);
+		g = lwgeom_from_wkt("LINESTRING(0 0,50 0.99999,100 0)", LW_PARSER_CHECK_NONE);
+		l = lwgeom_simplify(g, 1.0, LW_FALSE);
 		ewkt = lwgeom_to_ewkt(l);
 		CU_ASSERT_STRING_EQUAL(ewkt, "LINESTRING(0 0,100 0)");
+		lwgeom_free(g);
 		lwgeom_free(l);
-		lwgeom_free(i);
 		lwfree(ewkt);
 }
 
@@ -926,7 +965,7 @@ static void test_lwgeom_simplify(void)
 void algorithms_suite_setup(void);
 void algorithms_suite_setup(void)
 {
-	CU_pSuite suite = CU_add_suite("Computational Geometry", init_cg_suite, clean_cg_suite);
+	CU_pSuite suite = CU_add_suite("computational_geometry", init_cg_suite, clean_cg_suite);
 	PG_ADD_TEST(suite,test_lw_segment_side);
 	PG_ADD_TEST(suite,test_lw_segment_intersects);
 	PG_ADD_TEST(suite,test_lwline_crossing_short_lines);
