@@ -1,8 +1,7 @@
 /**********************************************************************
- * $Id: lwalgorithm.c 12043 2013-10-18 20:57:30Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
- * http://postgis.refractions.net
+ * http://postgis.net
  * Copyright 2008 Paul Ramsey
  *
  * This is free software; you can redistribute and/or modify it under
@@ -277,16 +276,15 @@ pt_in_ring_2d(const POINT2D *p, const POINTARRAY *ring)
 {
 	int cn = 0;    /* the crossing number counter */
 	int i;
-	POINT2D v1, v2;
+	const POINT2D *v1, *v2;
+	const POINT2D *first, *last;
 
-	POINT2D first, last;
-
-	getPoint2d_p(ring, 0, &first);
-	getPoint2d_p(ring, ring->npoints-1, &last);
-	if ( memcmp(&first, &last, sizeof(POINT2D)) )
+	first = getPoint2d_cp(ring, 0);
+	last = getPoint2d_cp(ring, ring->npoints-1);
+	if ( memcmp(first, last, sizeof(POINT2D)) )
 	{
 		lwerror("pt_in_ring_2d: V[n] != V[0] (%g %g != %g %g)",
-		        first.x, first.y, last.x, last.y);
+		        first->x, first->y, last->x, last->y);
 		return LW_FALSE;
 
 	}
@@ -295,28 +293,28 @@ pt_in_ring_2d(const POINT2D *p, const POINTARRAY *ring)
 	/* printPA(ring); */
 
 	/* loop through all edges of the polygon */
-	getPoint2d_p(ring, 0, &v1);
+	v1 = getPoint2d_cp(ring, 0);
 	for (i=0; i<ring->npoints-1; i++)
 	{
 		double vt;
-		getPoint2d_p(ring, i+1, &v2);
+		v2 = getPoint2d_cp(ring, i+1);
 
 		/* edge from vertex i to vertex i+1 */
 		if
 		(
 		    /* an upward crossing */
-		    ((v1.y <= p->y) && (v2.y > p->y))
+		    ((v1->y <= p->y) && (v2->y > p->y))
 		    /* a downward crossing */
-		    || ((v1.y > p->y) && (v2.y <= p->y))
+		    || ((v1->y > p->y) && (v2->y <= p->y))
 		)
 		{
 
-			vt = (double)(p->y - v1.y) / (v2.y - v1.y);
+			vt = (double)(p->y - v1->y) / (v2->y - v1->y);
 
-			/* P.x <intersect */
-			if (p->x < v1.x + vt * (v2.x - v1.x))
+			/* P->x <intersect */
+			if (p->x < v1->x + vt * (v2->x - v1->x))
 			{
-				/* a valid crossing of y=p.y right of p.x */
+				/* a valid crossing of y=p->y right of p->x */
 				++cn;
 			}
 		}
@@ -457,7 +455,7 @@ int lw_segment_intersects(const POINT2D *p1, const POINT2D *p2, const POINT2D *q
 int lwline_crossing_direction(const LWLINE *l1, const LWLINE *l2)
 {
 	int i = 0, j = 0;
-	POINT2D p1, p2, q1, q2;
+	const POINT2D *p1, *p2, *q1, *q2;
 	POINTARRAY *pa1 = NULL, *pa2 = NULL;
 	int cross_left = 0;
 	int cross_right = 0;
@@ -475,26 +473,26 @@ int lwline_crossing_direction(const LWLINE *l1, const LWLINE *l2)
 	LWDEBUGF(4, "l2 = %s", lwgeom_to_ewkt((LWGEOM*)l2));
 
 	/* Initialize first point of q */
-	getPoint2d_p(pa2, 0, &q1);
+	q1 = getPoint2d_cp(pa2, 0);
 
 	for ( i = 1; i < pa2->npoints; i++ )
 	{
 
 		/* Update second point of q to next value */
-		getPoint2d_p(pa2, i, &q2);
+		q2 = getPoint2d_cp(pa2, i);
 
 		/* Initialize first point of p */
-		getPoint2d_p(pa1, 0, &p1);
+		p1 = getPoint2d_cp(pa1, 0);
 
 		for ( j = 1; j < pa1->npoints; j++ )
 		{
 
 			/* Update second point of p to next value */
-			getPoint2d_p(pa1, j, &p2);
+			p2 = getPoint2d_cp(pa1, j);
 
-			this_cross = lw_segment_intersects(&p1, &p2, &q1, &q2);
+			this_cross = lw_segment_intersects(p1, p2, q1, q2);
 
-			LWDEBUGF(4, "i=%d, j=%d (%.8g %.8g, %.8g %.8g)", this_cross, i, j, p1.x, p1.y, p2.x, p2.y);
+			LWDEBUGF(4, "i=%d, j=%d (%.8g %.8g, %.8g %.8g)", this_cross, i, j, p1->x, p1->y, p2->x, p2->y);
 
 			if ( this_cross == SEG_CROSS_LEFT )
 			{
@@ -837,7 +835,9 @@ char *lwgeom_geohash(const LWGEOM *lwgeom, int precision)
 	/* Return error if we are being fed something outside our working bounds */
 	if ( gbox.xmin < -180 || gbox.ymin < -90 || gbox.xmax > 180 || gbox.ymax > 90 )
 	{
-		lwerror("Geohash requires inputs in decimal degrees.");
+		lwerror("Geohash requires inputs in decimal degrees, got (%g %g, %g %g).",
+			 gbox.xmin, gbox.ymin,
+			 gbox.xmax, gbox.ymax);
 		return NULL;
 	}
 
