@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: lwgeodetic.c 12674 2014-06-27 07:34:45Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -166,8 +165,8 @@ gbox_angular_height(const GBOX* gbox)
 {
 	double d[6];
 	int i;
-	double zmin = MAXFLOAT;
-	double zmax = -1 * MAXFLOAT;
+	double zmin = FLT_MAX;
+	double zmax = -1 * FLT_MAX;
 	POINT3D pt;
 	
 	/* Take a copy of the box corners so we can treat them as a list */
@@ -212,7 +211,7 @@ gbox_angular_width(const GBOX* gbox)
 	/* Find the vector furthest from our seed vector */
 	for ( j = 0; j < 2; j++ )
 	{
-		maxangle = -1 * MAXFLOAT;
+		maxangle = -1 * FLT_MAX;
 		for ( i = 0; i < 4; i++ )
 		{
 			double angle, dotprod;
@@ -297,7 +296,7 @@ static int gbox_check_poles(GBOX *gbox)
 	LWDEBUGF(4, "gbox %s", gbox_to_string(gbox));
 	/* Z axis */
 	if ( gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
-	        gbox->ymin < 0.0 && gbox->ymax > 0.0 )
+	     gbox->ymin < 0.0 && gbox->ymax > 0.0 )
 	{
 		if ( (gbox->zmin + gbox->zmax) > 0.0 )
 		{
@@ -314,7 +313,7 @@ static int gbox_check_poles(GBOX *gbox)
 
 	/* Y axis */
 	if ( gbox->xmin < 0.0 && gbox->xmax > 0.0 &&
-	        gbox->zmin < 0.0 && gbox->zmax > 0.0 )
+	     gbox->zmin < 0.0 && gbox->zmax > 0.0 )
 	{
 		if ( gbox->ymin + gbox->ymax > 0.0 )
 		{
@@ -331,7 +330,7 @@ static int gbox_check_poles(GBOX *gbox)
 
 	/* X axis */
 	if ( gbox->ymin < 0.0 && gbox->ymax > 0.0 &&
-	        gbox->zmin < 0.0 && gbox->zmax > 0.0 )
+	     gbox->zmin < 0.0 && gbox->zmax > 0.0 )
 	{
 		if ( gbox->xmin + gbox->xmax > 0.0 )
 		{
@@ -443,10 +442,12 @@ static void vector_scale(POINT3D *n, double scale)
 	return;
 }
 
-// static inline double vector_magnitude(const POINT3D* v)
-// {
-// 	return sqrt(v->x*v->x + v->y*v->y + v->z*v->z);
-// }
+/*
+* static inline double vector_magnitude(const POINT3D* v)
+* {
+*	return sqrt(v->x*v->x + v->y*v->y + v->z*v->z);
+* }
+*/
 
 /**
 * Angle between two unit vectors
@@ -806,7 +807,7 @@ int edge_contains_coplanar_point(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POIN
 			return LW_FALSE;
 
 		if ( ( g.start.lat <= q.lat && q.lat <= g.end.lat ) ||
-		        ( g.end.lat <= q.lat && q.lat <= g.start.lat ) )
+		     ( g.end.lat <= q.lat && q.lat <= g.start.lat ) )
 		{
 			return LW_TRUE;
 		}
@@ -880,7 +881,7 @@ int edge_contains_coplanar_point(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POIN
 	}
 
 	if ( ( g.start.lon <= q.lon && q.lon <= g.end.lon ) ||
-	        ( g.end.lon <= q.lon && q.lon <= g.start.lon ) )
+	     ( g.end.lon <= q.lon && q.lon <= g.start.lon ) )
 	{
 		LWDEBUG(4, "true, this edge contains point");
 		return LW_TRUE;
@@ -1171,7 +1172,7 @@ double edge_distance_to_point(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POINT *
 	/* Zero length edge, */
 	if ( geographic_point_equals(&(e->start), &(e->end)) )
 	{
-        *closest = e->start;
+		*closest = e->start;
 		return sphere_distance(&(e->start), gp);
 	}
 
@@ -1733,13 +1734,13 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 	GEOGRAPHIC_POINT g1, g2;
 	GEOGRAPHIC_POINT nearest1, nearest2;
 	POINT3D A1, A2, B1, B2;
-	POINT2D p;
+	const POINT2D *p;
 	double distance;
 	int i, j;
 	int use_sphere = (s->a == s->b ? 1 : 0);
 
 	/* Make result really big, so that everything will be smaller than it */
-	distance = MAXFLOAT;
+	distance = FLT_MAX;
 
 	/* Empty point arrays? Return negative */
 	if ( pa1->npoints == 0 || pa2->npoints == 0 )
@@ -1748,10 +1749,10 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 	/* Handle point/point case here */
 	if ( pa1->npoints == 1 && pa2->npoints == 1 )
 	{
-		getPoint2d_p(pa1, 0, &p);
-		geographic_point_init(p.x, p.y, &g1);
-		getPoint2d_p(pa2, 0, &p);
-		geographic_point_init(p.x, p.y, &g2);
+		p = getPoint2d_cp(pa1, 0);
+		geographic_point_init(p->x, p->y, &g1);
+		p = getPoint2d_cp(pa2, 0);
+		geographic_point_init(p->x, p->y, &g2);
 		/* Sphere special case, axes equal */
 		distance = s->radius * sphere_distance(&g1, &g2);
 		if ( use_sphere )
@@ -1784,19 +1785,19 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 		}
 
 		/* Initialize our point */
-		getPoint2d_p(pa_one, 0, &p);
-		geographic_point_init(p.x, p.y, &g1);
+		p = getPoint2d_cp(pa_one, 0);
+		geographic_point_init(p->x, p->y, &g1);
 
 		/* Initialize start of line */
-		getPoint2d_p(pa_many, 0, &p);
-		geographic_point_init(p.x, p.y, &(e1.start));
+		p = getPoint2d_cp(pa_many, 0);
+		geographic_point_init(p->x, p->y, &(e1.start));
 
 		/* Iterate through the edges in our line */
 		for ( i = 1; i < pa_many->npoints; i++ )
 		{
 			double d;
-			getPoint2d_p(pa_many, i, &p);
-			geographic_point_init(p.x, p.y, &(e1.end));
+			p = getPoint2d_cp(pa_many, i);
+			geographic_point_init(p->x, p->y, &(e1.end));
 			/* Get the spherical distance between point and edge */
 			d = s->radius * edge_distance_to_point(&e1, &g1, &g2);
 			/* New shortest distance! Record this distance / location */
@@ -1836,32 +1837,33 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 		/* On spheroid, calculate final answer based on closest approach */
 		else
 			return spheroid_distance(&g1, &nearest2, s);
+
 	}
 
 	/* Initialize start of line 1 */
-	getPoint2d_p(pa1, 0, &p);
-	geographic_point_init(p.x, p.y, &(e1.start));
+	p = getPoint2d_cp(pa1, 0);
+	geographic_point_init(p->x, p->y, &(e1.start));
 	geog2cart(&(e1.start), &A1);
 
 
 	/* Handle line/line case */
 	for ( i = 1; i < pa1->npoints; i++ )
 	{
-		getPoint2d_p(pa1, i, &p);
-		geographic_point_init(p.x, p.y, &(e1.end));
+		p = getPoint2d_cp(pa1, i);
+		geographic_point_init(p->x, p->y, &(e1.end));
 		geog2cart(&(e1.end), &A2);
 
 		/* Initialize start of line 2 */
-		getPoint2d_p(pa2, 0, &p);
-		geographic_point_init(p.x, p.y, &(e2.start));
+		p = getPoint2d_cp(pa2, 0);
+		geographic_point_init(p->x, p->y, &(e2.start));
 		geog2cart(&(e2.start), &B1);
 
 		for ( j = 1; j < pa2->npoints; j++ )
 		{
 			double d;
 
-			getPoint2d_p(pa2, j, &p);
-			geographic_point_init(p.x, p.y, &(e2.end));
+			p = getPoint2d_cp(pa2, j);
+			geographic_point_init(p->x, p->y, &(e2.end));
 			geog2cart(&(e2.end), &B2);
 
 			LWDEBUGF(4, "e1.start == GPOINT(%.6g %.6g) ", e1.start.lat, e1.start.lon);
@@ -2117,7 +2119,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 
 	/* Point/line combinations can all be handled with simple point array iterations */
 	if ( ( type1 == POINTTYPE || type1 == LINETYPE ) &&
-	        ( type2 == POINTTYPE || type2 == LINETYPE ) )
+	     ( type2 == POINTTYPE || type2 == LINETYPE ) )
 	{
 		POINTARRAY *pa1, *pa2;
 
@@ -2136,12 +2138,12 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 
 	/* Point/Polygon cases, if point-in-poly, return zero, else return distance. */
 	if ( ( type1 == POLYGONTYPE && type2 == POINTTYPE ) ||
-	        ( type2 == POLYGONTYPE && type1 == POINTTYPE ) )
+	     ( type2 == POLYGONTYPE && type1 == POINTTYPE ) )
 	{
-		POINT2D p;
+		const POINT2D *p;
 		LWPOLY *lwpoly;
 		LWPOINT *lwpt;
-		double distance = MAXFLOAT;
+		double distance = FLT_MAX;
 		int i;
 
 		if ( type1 == POINTTYPE )
@@ -2154,10 +2156,10 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			lwpt = (LWPOINT*)lwgeom2;
 			lwpoly = (LWPOLY*)lwgeom1;
 		}
-		getPoint2d_p(lwpt->point, 0, &p);
+		p = getPoint2d_cp(lwpt->point, 0);
 
 		/* Point in polygon implies zero distance */
-		if ( lwpoly_covers_point2d(lwpoly, &p) )
+		if ( lwpoly_covers_point2d(lwpoly, p) )
 		{
 			return 0.0;
 		}
@@ -2176,12 +2178,12 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 
 	/* Line/polygon case, if start point-in-poly, return zero, else return distance. */
 	if ( ( type1 == POLYGONTYPE && type2 == LINETYPE ) ||
-	        ( type2 == POLYGONTYPE && type1 == LINETYPE ) )
+	     ( type2 == POLYGONTYPE && type1 == LINETYPE ) )
 	{
-		POINT2D p;
+		const POINT2D *p;
 		LWPOLY *lwpoly;
 		LWLINE *lwline;
-		double distance = MAXFLOAT;
+		double distance = FLT_MAX;
 		int i;
 
 		if ( type1 == LINETYPE )
@@ -2194,12 +2196,12 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			lwline = (LWLINE*)lwgeom2;
 			lwpoly = (LWPOLY*)lwgeom1;
 		}
-		getPoint2d_p(lwline->points, 0, &p);
+		p = getPoint2d_cp(lwline->points, 0);
 
 		LWDEBUG(4, "checking if a point of line is in polygon");
 
 		/* Point in polygon implies zero distance */
-		if ( lwpoly_covers_point2d(lwpoly, &p) ) 
+		if ( lwpoly_covers_point2d(lwpoly, p) ) 
 			return 0.0;
 
 		LWDEBUG(4, "checking ring distances");
@@ -2221,22 +2223,22 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 
 	/* Polygon/polygon case, if start point-in-poly, return zero, else return distance. */
 	if ( ( type1 == POLYGONTYPE && type2 == POLYGONTYPE ) ||
-	        ( type2 == POLYGONTYPE && type1 == POLYGONTYPE ) )
+	     ( type2 == POLYGONTYPE && type1 == POLYGONTYPE ) )
 	{
-		POINT2D p;
+		const POINT2D *p;
 		LWPOLY *lwpoly1 = (LWPOLY*)lwgeom1;
 		LWPOLY *lwpoly2 = (LWPOLY*)lwgeom2;
-		double distance = MAXFLOAT;
+		double distance = FLT_MAX;
 		int i, j;
 
 		/* Point of 2 in polygon 1 implies zero distance */
-		getPoint2d_p(lwpoly1->rings[0], 0, &p);
-		if ( lwpoly_covers_point2d(lwpoly2, &p) )
+		p = getPoint2d_cp(lwpoly1->rings[0], 0);
+		if ( lwpoly_covers_point2d(lwpoly2, p) )
 			return 0.0;
 
 		/* Point of 1 in polygon 2 implies zero distance */
-		getPoint2d_p(lwpoly2->rings[0], 0, &p);
-		if ( lwpoly_covers_point2d(lwpoly1, &p) )
+		p = getPoint2d_cp(lwpoly2->rings[0], 0);
+		if ( lwpoly_covers_point2d(lwpoly1, p) )
 			return 0.0;
 
 		/* Not contained, so what's the actual distance? */
@@ -2258,7 +2260,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 	if ( lwtype_is_collection(type1) )
 	{
 		int i;
-		double distance = MAXFLOAT;
+		double distance = FLT_MAX;
 		LWCOLLECTION *col = (LWCOLLECTION*)lwgeom1;
 
 		for ( i = 0; i < col->ngeoms; i++ )
@@ -2276,7 +2278,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 	if ( lwtype_is_collection(type2) )
 	{
 		int i;
-		double distance = MAXFLOAT;
+		double distance = FLT_MAX;
 		LWCOLLECTION *col = (LWCOLLECTION*)lwgeom2;
 
 		for ( i = 0; i < col->ngeoms; i++ )
