@@ -317,6 +317,50 @@ lwgeom_as_multi(const LWGEOM *lwgeom)
 	return ogeom;
 }
 
+/**
+* Create a new LWGEOM of the appropriate CURVE* type.
+*/
+LWGEOM *
+lwgeom_as_curve(const LWGEOM *lwgeom)
+{
+	LWGEOM *ogeom;
+	int type = lwgeom->type;
+	/*
+	int hasz = FLAGS_GET_Z(lwgeom->flags);
+	int hasm = FLAGS_GET_M(lwgeom->flags);
+	int srid = lwgeom->srid;
+	*/
+
+	switch(type)
+	{
+		case LINETYPE:
+			/* turn to COMPOUNDCURVE */
+			ogeom = (LWGEOM*)lwcompound_construct_from_lwline((LWLINE*)lwgeom);
+			break;
+		case POLYGONTYPE:
+			ogeom = (LWGEOM*)lwcurvepoly_construct_from_lwpoly(lwgeom_as_lwpoly(lwgeom));
+			break;
+		case MULTILINETYPE:
+			/* turn to MULTICURVE */
+			ogeom = lwgeom_clone(lwgeom);
+			ogeom->type = MULTICURVETYPE;
+			break;
+		case MULTIPOLYGONTYPE:
+			/* turn to MULTISURFACE */
+			ogeom = lwgeom_clone(lwgeom);
+			ogeom->type = MULTISURFACETYPE;
+			break;
+		case COLLECTIONTYPE:
+		default:
+			ogeom = lwgeom_clone(lwgeom);
+			break;
+	}
+
+	/* TODO: copy bbox from input geom ? */
+
+	return ogeom;
+}
+
 
 /**
 * Free the containing LWGEOM and the associated BOX. Leave the underlying 
@@ -521,18 +565,18 @@ lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 int
 lwpoint_inside_circle(const LWPOINT *p, double cx, double cy, double rad)
 {
+	const POINT2D *pt;
 	POINT2D center;
-	POINT2D pt;
 
 	if ( ! p || ! p->point )
 		return LW_FALSE;
 		
-	getPoint2d_p(p->point, 0, &pt);
+	pt = getPoint2d_cp(p->point, 0);
 
 	center.x = cx;
 	center.y = cy;
 
-	if ( distance2d_pt_pt(&pt, &center) < rad ) 
+	if ( distance2d_pt_pt(pt, &center) < rad ) 
 		return LW_TRUE;
 
 	return LW_FALSE;
@@ -714,7 +758,7 @@ lwgeom_force_sfs(LWGEOM *geom, int version)
 			case CURVEPOLYTYPE:
 			case MULTICURVETYPE:
 			case MULTISURFACETYPE:
-				return lwgeom_segmentize(geom, 32);
+				return lwgeom_stroke(geom, 32);
 
 			case COLLECTIONTYPE:
 				col = (LWCOLLECTION*)geom;
@@ -738,7 +782,7 @@ lwgeom_force_sfs(LWGEOM *geom, int version)
 		case CURVEPOLYTYPE:
 		case MULTICURVETYPE:
 		case MULTISURFACETYPE:
-			return lwgeom_segmentize(geom, 32);
+			return lwgeom_stroke(geom, 32);
 
 		/* SFS 1.2 types */
 		case TRIANGLETYPE:
@@ -1063,7 +1107,31 @@ int lwgeom_needs_bbox(const LWGEOM *geom)
 	{
 		return LW_FALSE;
 	}
-	return LW_TRUE;
+	else if ( geom->type == LINETYPE )
+	{
+		if ( lwgeom_count_vertices(geom) <= 2 )
+			return LW_FALSE;
+		else
+			return LW_TRUE;
+	}
+	else if ( geom->type == MULTIPOINTTYPE )
+	{
+		if ( ((LWCOLLECTION*)geom)->ngeoms == 1 )
+			return LW_FALSE;
+		else
+			return LW_TRUE;
+	}
+	else if ( geom->type == MULTILINETYPE )
+	{
+		if ( ((LWCOLLECTION*)geom)->ngeoms == 1 && lwgeom_count_vertices(geom) <= 2 )
+			return LW_FALSE;
+		else
+			return LW_TRUE;
+	}
+	else
+	{
+		return LW_TRUE;
+	}
 }
 
 /**
@@ -1108,8 +1176,8 @@ int lwgeom_count_vertices(const LWGEOM *geom)
 		result = lwcollection_count_vertices((LWCOLLECTION *)geom);
 		break;
 	default:
-		lwerror("lwgeom_count_vertices: unsupported input geometry type: %s",
-		        lwtype_name(geom->type));
+		lwerror("%s: unsupported input geometry type: %s",
+		        __func__, lwtype_name(geom->type));
 		break;
 	}
 	LWDEBUGF(3, "counted %d vertices", result);
@@ -1169,8 +1237,8 @@ int lwgeom_dimension(const LWGEOM *geom)
 		return maxdim;
 	}
 	default:
-		lwerror("lwgeom_dimension: unsupported input geometry type: %s",
-		        lwtype_name(geom->type));
+		lwerror("%s: unsupported input geometry type: %s",
+		        __func__, lwtype_name(geom->type));
 	}
 	return -1;
 }
@@ -1336,7 +1404,7 @@ extern int lwgeom_dimensionality(LWGEOM *geom)
 	return 0;
 }
 
-extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in)
+extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance)
 {
 	LWDEBUGF(4, "lwgeom_remove_repeated_points got type %s",
 	         lwtype_name(in->type));
@@ -1344,19 +1412,19 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in)
 	switch (in->type)
 	{
 	case MULTIPOINTTYPE:
-		return lwmpoint_remove_repeated_points((LWMPOINT*)in);
+		return lwmpoint_remove_repeated_points((LWMPOINT*)in, tolerance);
 		break;
 	case LINETYPE:
-		return lwline_remove_repeated_points((LWLINE*)in);
+		return lwline_remove_repeated_points((LWLINE*)in, tolerance);
 
 	case MULTILINETYPE:
 	case COLLECTIONTYPE:
 	case MULTIPOLYGONTYPE:
 	case POLYHEDRALSURFACETYPE:
-		return lwcollection_remove_repeated_points((LWCOLLECTION *)in);
+		return lwcollection_remove_repeated_points((LWCOLLECTION *)in, tolerance);
 
 	case POLYGONTYPE:
-		return lwpoly_remove_repeated_points((LWPOLY *)in);
+		return lwpoly_remove_repeated_points((LWPOLY *)in, tolerance);
 		break;
 
 	case POINTTYPE:
@@ -1374,8 +1442,8 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in)
 		return in;
 
 	default:
-		lwnotice("lwgeom_remove_repeated_points: unsupported geometry type: %s",
-		         lwtype_name(in->type));
+		lwnotice("%s: unsupported geometry type: %s",
+		         __func__, lwtype_name(in->type));
 		return in;
 		break;
 	}
@@ -1383,13 +1451,25 @@ extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in)
 }
 
 LWGEOM* lwgeom_flip_coordinates(LWGEOM *in)
+{
+  lwgeom_swap_ordinates(in,LWORD_X,LWORD_Y);
+  return in;
+}
+
+void lwgeom_swap_ordinates(LWGEOM *in, LWORD o1, LWORD o2)
 {
 	LWCOLLECTION *col;
 	LWPOLY *poly;
 	int i;
 
-	if ( (!in) || lwgeom_is_empty(in) )
-		return in;
+#if PARANOIA_LEVEL > 0
+  assert(o1 < 4);
+  assert(o2 < 4);
+#endif
+
+	if ( (!in) || lwgeom_is_empty(in) ) return;
+
+  /* TODO: check for lwgeom NOT having the specified dimension ? */
 
 	LWDEBUGF(4, "lwgeom_flip_coordinates, got type: %s",
 	         lwtype_name(in->type));
@@ -1397,27 +1477,27 @@ LWGEOM* lwgeom_flip_coordinates(LWGEOM *in)
 	switch (in->type)
 	{
 	case POINTTYPE:
-		ptarray_flip_coordinates(lwgeom_as_lwpoint(in)->point);
+		ptarray_swap_ordinates(lwgeom_as_lwpoint(in)->point, o1, o2);
 		break;
 
 	case LINETYPE:
-		ptarray_flip_coordinates(lwgeom_as_lwline(in)->points);
+		ptarray_swap_ordinates(lwgeom_as_lwline(in)->points, o1, o2);
 		break;
 
 	case CIRCSTRINGTYPE:
-		ptarray_flip_coordinates(lwgeom_as_lwcircstring(in)->points);
+		ptarray_swap_ordinates(lwgeom_as_lwcircstring(in)->points, o1, o2);
 		break;
 
 	case POLYGONTYPE:
 		poly = (LWPOLY *) in;
 		for (i=0; i<poly->nrings; i++)
 		{
-			ptarray_flip_coordinates(poly->rings[i]);
+			ptarray_swap_ordinates(poly->rings[i], o1, o2);
 		}
 		break;
 
 	case TRIANGLETYPE:
-		ptarray_flip_coordinates(lwgeom_as_lwtriangle(in)->points);
+		ptarray_swap_ordinates(lwgeom_as_lwtriangle(in)->points, o1, o2);
 		break;
 
 	case MULTIPOINTTYPE:
@@ -1433,22 +1513,22 @@ LWGEOM* lwgeom_flip_coordinates(LWGEOM *in)
 		col = (LWCOLLECTION *) in;
 		for (i=0; i<col->ngeoms; i++)
 		{
-			lwgeom_flip_coordinates(col->geoms[i]);
+			lwgeom_swap_ordinates(col->geoms[i], o1, o2);
 		}
 		break;
 
 	default:
-		lwerror("lwgeom_flip_coordinates: unsupported geometry type: %s",
+		lwerror("lwgeom_swap_ordinates: unsupported geometry type: %s",
 		        lwtype_name(in->type));
-		return NULL;
+		return;
 	}
 
-  /* only refresh bbox if X or Y changed */
-	if ( in->bbox ) {
+	/* only refresh bbox if X or Y changed */
+	if ( in->bbox && (o1 < 2 || o2 < 2) ) 
+	{
 		lwgeom_drop_bbox(in);
 		lwgeom_add_bbox(in);
 	}
-	return in;
 }
 
 void lwgeom_set_srid(LWGEOM *geom, int32_t srid)
@@ -1470,7 +1550,7 @@ void lwgeom_set_srid(LWGEOM *geom, int32_t srid)
 	}
 }
 
-LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist)
+LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist, int preserve_collapsed)
 {
 	switch (igeom->type)
 	{
@@ -1478,15 +1558,15 @@ LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist)
 	case MULTIPOINTTYPE:
 		return lwgeom_clone(igeom);
 	case LINETYPE:
-		return (LWGEOM*)lwline_simplify((LWLINE*)igeom, dist);
+		return (LWGEOM*)lwline_simplify((LWLINE*)igeom, dist, preserve_collapsed);
 	case POLYGONTYPE:
-		return (LWGEOM*)lwpoly_simplify((LWPOLY*)igeom, dist);
+		return (LWGEOM*)lwpoly_simplify((LWPOLY*)igeom, dist, preserve_collapsed);
 	case MULTILINETYPE:
 	case MULTIPOLYGONTYPE:
 	case COLLECTIONTYPE:
-		return (LWGEOM*)lwcollection_simplify((LWCOLLECTION *)igeom, dist);
+		return (LWGEOM*)lwcollection_simplify((LWCOLLECTION *)igeom, dist, preserve_collapsed);
 	default:
-		lwerror("lwgeom_simplify: unsupported geometry type: %s",lwtype_name(igeom->type));
+		lwerror("%s: unsupported geometry type: %s", __func__, lwtype_name(igeom->type));
 	}
 	return NULL;
 }
@@ -1653,6 +1733,71 @@ lwgeom_affine(LWGEOM *geom, const AFFINE *affine)
 
 }
 
+void
+lwgeom_scale(LWGEOM *geom, const POINT4D *factor)
+{
+	int type = geom->type;
+	int i;
+
+	switch(type) 
+	{
+		/* Take advantage of fact tht pt/ln/circ/tri have same memory structure */
+		case POINTTYPE:
+		case LINETYPE:
+		case CIRCSTRINGTYPE:
+		case TRIANGLETYPE:
+		{
+			LWLINE *l = (LWLINE*)geom;
+			ptarray_scale(l->points, factor);
+			break;
+		}
+		case POLYGONTYPE:
+		{
+			LWPOLY *p = (LWPOLY*)geom;
+			for( i = 0; i < p->nrings; i++ )
+				ptarray_scale(p->rings[i], factor);
+			break;
+		}
+		case CURVEPOLYTYPE:
+		{
+			LWCURVEPOLY *c = (LWCURVEPOLY*)geom;
+			for( i = 0; i < c->nrings; i++ )
+				lwgeom_scale(c->rings[i], factor);
+			break;
+		}
+		default:
+		{
+			if( lwgeom_is_collection(geom) )
+			{
+				LWCOLLECTION *c = (LWCOLLECTION*)geom;
+				for( i = 0; i < c->ngeoms; i++ )
+				{
+					lwgeom_scale(c->geoms[i], factor);
+				}
+			}
+			else 
+			{
+				lwerror("lwgeom_scale: unable to handle type '%s'", lwtype_name(type));
+			}
+		}
+	}
+
+	/* Recompute bbox if needed */
+
+	if ( geom->bbox ) 
+	{
+		/* TODO: expose a gbox_scale function */
+		geom->bbox->xmin *= factor->x;
+		geom->bbox->xmax *= factor->x;
+		geom->bbox->ymin *= factor->y;
+		geom->bbox->ymax *= factor->y;
+		geom->bbox->zmin *= factor->z;
+		geom->bbox->zmax *= factor->z;
+		geom->bbox->mmin *= factor->m;
+		geom->bbox->mmax *= factor->m;
+	}
+}
+
 LWGEOM*
 lwgeom_construct_empty(uint8_t type, int srid, char hasz, char hasm)
 {
@@ -1713,3 +1858,173 @@ lwgeom_startpoint(const LWGEOM* lwgeom, POINT4D* pt)
 	}
 }
 
+
+LWGEOM *
+lwgeom_grid(const LWGEOM *lwgeom, const gridspec *grid)
+{
+	switch ( lwgeom->type )
+	{
+		case POINTTYPE:
+			return (LWGEOM *)lwpoint_grid((LWPOINT *)lwgeom, grid);
+		case LINETYPE:
+			return (LWGEOM *)lwline_grid((LWLINE *)lwgeom, grid);
+		case POLYGONTYPE:
+			return (LWGEOM *)lwpoly_grid((LWPOLY *)lwgeom, grid);
+		case MULTIPOINTTYPE:
+		case MULTILINETYPE:
+		case MULTIPOLYGONTYPE:
+		case COLLECTIONTYPE:
+		case COMPOUNDTYPE:
+			return (LWGEOM *)lwcollection_grid((LWCOLLECTION *)lwgeom, grid);
+		case CIRCSTRINGTYPE:
+			return (LWGEOM *)lwcircstring_grid((LWCIRCSTRING *)lwgeom, grid);
+		default:
+			lwerror("lwgeom_grid: Unsupported geometry type: %s",
+			        lwtype_name(lwgeom->type));
+			return NULL;
+	}
+}
+
+
+/* Prototype for recursion */
+static int 
+lwgeom_subdivide_recursive(const LWGEOM *geom, int maxvertices, int depth, LWCOLLECTION *col, const GBOX *clip);
+
+static int
+lwgeom_subdivide_recursive(const LWGEOM *geom, int maxvertices, int depth, LWCOLLECTION *col, const GBOX *clip)
+{
+	const int maxdepth = 50;
+	int nvertices = 0;
+	int i, n = 0;
+	double width = clip->xmax - clip->xmin;
+	double height = clip->ymax - clip->ymin;
+	GBOX subbox1, subbox2;
+	LWGEOM *clipped1, *clipped2;
+	
+	if ( geom->type == POLYHEDRALSURFACETYPE || geom->type == TINTYPE )
+	{
+		lwerror("%s: unsupported geometry type '%s'", __func__, lwtype_name(geom->type));
+	}
+	
+	if ( width == 0.0 && height == 0.0 )
+		return 0;
+	
+	/* Always just recurse into collections */
+	if ( lwgeom_is_collection(geom) )
+	{
+		LWCOLLECTION *incol = (LWCOLLECTION*)geom;
+		int n = 0;
+		for ( i = 0; i < incol->ngeoms; i++ )
+		{
+			/* Don't increment depth yet, since we aren't actually subdividing geomtries yet */
+			n += lwgeom_subdivide_recursive(incol->geoms[i], maxvertices, depth, col, clip);
+		}
+		return n;
+	}
+	
+	/* But don't go too far. 2^25 = 33M, that's enough subdivision */
+	/* Signal callers above that we depth'ed out with a negative */
+	/* return value */
+	if ( depth > maxdepth )
+	{
+		return 0;
+	}
+	
+	nvertices = lwgeom_count_vertices(geom);
+	/* Skip empties entirely */
+	if ( nvertices == 0 )
+	{
+		return 0;
+	}
+	
+	/* If it is under the vertex tolerance, just add it, we're done */
+	if ( nvertices < maxvertices )
+	{
+		lwcollection_add_lwgeom(col, lwgeom_clone_deep(geom));
+		return 1;
+	}
+	
+	subbox1 = subbox2 = *clip;
+	if ( width > height )
+	{
+		subbox1.xmax = subbox2.xmin = (clip->xmin + clip->xmax)/2;
+	}
+	else
+	{
+		subbox1.ymax = subbox2.ymin = (clip->ymin + clip->ymax)/2;
+	}
+	
+	if ( height == 0 )
+	{
+		subbox1.ymax += FP_TOLERANCE;
+		subbox2.ymax += FP_TOLERANCE;
+		subbox1.ymin -= FP_TOLERANCE;
+		subbox2.ymin -= FP_TOLERANCE;
+	}
+
+	if ( width == 0 )
+	{
+		subbox1.xmax += FP_TOLERANCE;
+		subbox2.xmax += FP_TOLERANCE;
+		subbox1.xmin -= FP_TOLERANCE;
+		subbox2.xmin -= FP_TOLERANCE;
+	}
+		
+	clipped1 = lwgeom_clip_by_rect(geom, subbox1.xmin, subbox1.ymin, subbox1.xmax, subbox1.ymax);
+	clipped2 = lwgeom_clip_by_rect(geom, subbox2.xmin, subbox2.ymin, subbox2.xmax, subbox2.ymax);
+	
+	if ( clipped1 )
+	{
+		n += lwgeom_subdivide_recursive(clipped1, maxvertices, ++depth, col, &subbox1);
+		lwgeom_free(clipped1);
+	}
+
+	if ( clipped2 )
+	{
+		n += lwgeom_subdivide_recursive(clipped2, maxvertices, ++depth, col, &subbox2);
+		lwgeom_free(clipped2);
+	}
+	
+	return n;
+	
+}
+
+LWCOLLECTION *
+lwgeom_subdivide(const LWGEOM *geom, int maxvertices)
+{
+	static int startdepth = 0;
+	static int minmaxvertices = 8;
+	LWCOLLECTION *col;
+	GBOX clip;
+	
+	col = lwcollection_construct_empty(COLLECTIONTYPE, geom->srid, lwgeom_has_z(geom), lwgeom_has_m(geom));
+
+	if ( lwgeom_is_empty(geom) )
+		return col;
+
+	if ( maxvertices < minmaxvertices )
+	{
+		lwcollection_free(col);
+		lwerror("%s: cannot subdivide to fewer than %d vertices per output", __func__, minmaxvertices);
+	}
+	
+	clip = *(lwgeom_get_bbox(geom));
+	lwgeom_subdivide_recursive(geom, maxvertices, startdepth, col, &clip);
+	lwgeom_set_srid((LWGEOM*)col, geom->srid);
+	return col;
+}
+
+
+int
+lwgeom_is_trajectory(const LWGEOM *geom)
+{
+	int type = geom->type;
+
+	if( type != LINETYPE ) 
+	{
+		lwnotice("Geometry is not a LINESTRING");
+		return LW_FALSE;
+	}
+	return lwline_is_trajectory((LWLINE*)geom);
+}
+
