@@ -31,7 +31,7 @@
 #define LWDEBUG(level, msg) \
         do { \
             if (POSTGIS_DEBUG_LEVEL >= level) \
-              lwnotice("[%s:%s:%d] " msg, __FILE__, __func__, __LINE__); \
+              lwdebug(level, "[%s:%s:%d] " msg, __FILE__, __func__, __LINE__); \
         } while (0);
 
 /* Display a formatted notice at the given debug level
@@ -39,7 +39,7 @@
 #define LWDEBUGF(level, msg, ...) \
         do { \
             if (POSTGIS_DEBUG_LEVEL >= level) \
-              lwnotice("[%s:%s:%d] " msg, \
+              lwdebug(level, "[%s:%s:%d] " msg, \
                 __FILE__, __func__, __LINE__, __VA_ARGS__); \
         } while (0);
 
@@ -57,4 +57,35 @@
 
 #endif /* POSTGIS_DEBUG_LEVEL <= 0 */
 
+/**
+ * Write a notice out to the notice handler.
+ *
+ * Uses standard printf() substitutions.
+ * Use for messages you always want output.
+ * For debugging, use LWDEBUG() or LWDEBUGF().
+ * @ingroup logging
+ */
+void lwnotice(const char *fmt, ...);
+
+/**
+ * Write a notice out to the error handler.
+ *
+ * Uses standard printf() substitutions.
+ * Use for errors you always want output.
+ * For debugging, use LWDEBUG() or LWDEBUGF().
+ * @ingroup logging
+ */
+void lwerror(const char *fmt, ...);
+
+/**
+ * Write a debug message out. 
+ * Don't call this function directly, use the 
+ * macros, LWDEBUG() or LWDEBUGF(), for
+ * efficiency.
+ * @ingroup logging
+ */
+void lwdebug(int level, const char *fmt, ...);
+
+
+
 #endif /* LWGEOM_LOG_H */
