@@ -15,9 +15,7 @@
 #include "lwgeom_log.h"
 #include "../postgis_config.h"
 
-#ifdef HAVE_LIBJSON
-
-#include <string.h>
+#if defined(HAVE_LIBJSON) || defined(HAVE_LIBJSON_C) /* --{ */
 
 #ifdef HAVE_LIBJSON_C
 #include <json-c/json.h>
@@ -28,10 +26,11 @@
 #endif
 
 #ifndef JSON_C_VERSION
-// Adds support for libjson < 0.10
+/* Adds support for libjson < 0.10 */
 # define json_tokener_error_desc(x) json_tokener_errors[(x)]
 #endif
 
+#include <string.h>
 
 static void geojson_lwerror(char *msg, int error_code)
 {
@@ -40,7 +39,7 @@ static void geojson_lwerror(char *msg, int error_code)
 }
 
 /* Prototype */
-static LWGEOM* parse_geojson(json_object *geojson, int *hasz,  int root_srid);
+static LWGEOM* parse_geojson(json_object *geojson, int *hasz, int root_srid);
 
 static json_object*
 findMemberByName(json_object* poObj, const char* pszName )
@@ -100,19 +99,19 @@ parse_geojson_coord(json_object *poObj, int *hasz, POINTARRAY *pa)
 			return LW_FAILURE;
 		}
 		
-		// Read X coordinate
+		/* Read X coordinate */
 		poObjCoord = json_object_array_get_idx( poObj, 0 );
 		pt.x = json_object_get_double( poObjCoord );
 		LWDEBUGF(3, "parse_geojson_coord pt.x = %f.", pt.x );
 
-		// Read Y coordinate
+		/* Read Y coordinate */
 		poObjCoord = json_object_array_get_idx( poObj, 1 );
 		pt.y = json_object_get_double( poObjCoord );
 		LWDEBUGF(3, "parse_geojson_coord pt.y = %f.", pt.y );
 
 		if( nSize > 2 ) /* should this be >= 3 ? */
 		{
-			// Read Z coordinate
+			/* Read Z coordinate */
 			poObjCoord = json_object_array_get_idx( poObj, 2 );
 			pt.z = json_object_get_double( poObjCoord );
 			LWDEBUGF(3, "parse_geojson_coord pt.z = %f.", pt.z );
@@ -144,7 +143,7 @@ parse_geojson_coord(json_object *poObj, int *hasz, POINTARRAY *pa)
 }
 
 static LWGEOM*
-parse_geojson_point(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_point(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom;
 	POINTARRAY *pa;
@@ -168,7 +167,7 @@ parse_geojson_point(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_linestring(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_linestring(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom;
 	POINTARRAY *pa;
@@ -178,9 +177,10 @@ parse_geojson_linestring(json_object *geojson, int *hasz,  int root_srid)
 	LWDEBUG(2, "parse_geojson_linestring called.");
 
 	points = findMemberByName( geojson, "coordinates" );
-	if ( ! points ) {
+	if ( ! points ) 
+	{
 		geojson_lwerror("Unable to find 'coordinates' in GeoJSON string", 4);
-		return NULL;
+	return NULL;
 	}
 
 	pa = ptarray_construct_empty(1, 0, 1);
@@ -203,7 +203,7 @@ parse_geojson_linestring(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_polygon(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_polygon(json_object *geojson, int *hasz, int root_srid)
 {
 	POINTARRAY **ppa = NULL;
 	json_object* rings = NULL;
@@ -265,7 +265,7 @@ parse_geojson_polygon(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_multipoint(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_multipoint(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom;
 	int i = 0;
@@ -281,10 +281,11 @@ parse_geojson_multipoint(json_object *geojson, int *hasz,  int root_srid)
 	}
 
 	poObjPoints = findMemberByName( geojson, "coordinates" );
-	if ( ! poObjPoints ) {
+	if ( ! poObjPoints ) 
+	{
 		geojson_lwerror("Unable to find 'coordinates' in GeoJSON string", 4);
-    return NULL;
-  }
+		return NULL;
+	}
 
 	if( json_type_array == json_object_get_type( poObjPoints ) )
 	{
@@ -307,7 +308,7 @@ parse_geojson_multipoint(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_multilinestring(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_multilinestring(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom = NULL;
 	int i, j;
@@ -323,10 +324,11 @@ parse_geojson_multilinestring(json_object *geojson, int *hasz,  int root_srid)
 	}
 
 	poObjLines = findMemberByName( geojson, "coordinates" );
-	if ( ! poObjLines ) {
+	if ( ! poObjLines ) 
+	{
 		geojson_lwerror("Unable to find 'coordinates' in GeoJSON string", 4);
-    return NULL;
-  }
+		return NULL;
+	}
 
 	if( json_type_array == json_object_get_type( poObjLines ) )
 	{
@@ -358,7 +360,7 @@ parse_geojson_multilinestring(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_multipolygon(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_multipolygon(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom = NULL;
 	int i, j, k;
@@ -374,7 +376,7 @@ parse_geojson_multipolygon(json_object *geojson, int *hasz,  int root_srid)
 	}
 
 	poObjPolys = findMemberByName( geojson, "coordinates" );
-	if ( ! poObjPolys )
+	if ( ! poObjPolys ) 
 	{
 		geojson_lwerror("Unable to find 'coordinates' in GeoJSON string", 4);
 		return NULL;
@@ -421,7 +423,7 @@ parse_geojson_multipolygon(json_object *geojson, int *hasz,  int root_srid)
 }
 
 static LWGEOM*
-parse_geojson_geometrycollection(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson_geometrycollection(json_object *geojson, int *hasz, int root_srid)
 {
 	LWGEOM *geom = NULL;
 	int i;
@@ -459,7 +461,7 @@ parse_geojson_geometrycollection(json_object *geojson, int *hasz,  int root_srid
 }
 
 static LWGEOM*
-parse_geojson(json_object *geojson, int *hasz,  int root_srid)
+parse_geojson(json_object *geojson, int *hasz, int root_srid)
 {
 	json_object* type = NULL;
 	const char* name;
@@ -504,16 +506,16 @@ parse_geojson(json_object *geojson, int *hasz,  int root_srid)
 	return NULL; /* Never reach */
 }
 
-#endif /* HAVE_LIBJSON  */
+#endif /* HAVE_LIBJSON or HAVE_LIBJSON_C --} */
 
 LWGEOM*
 lwgeom_from_geojson(const char *geojson, char **srs)
 {
 #ifndef HAVE_LIBJSON
-  *srs = NULL;
+	*srs = NULL;
 	lwerror("You need JSON-C for lwgeom_from_geojson");
 	return NULL;
-#else /* HAVE_LIBJSON  */
+#else /* HAVE_LIBJSON */
 
 	/* size_t geojson_size = strlen(geojson); */
 
@@ -575,8 +577,8 @@ lwgeom_from_geojson(const char *geojson, char **srs)
 		LWDEBUG(2, "geom_from_geojson called.");
 	}
 
-  return lwgeom;
-#endif /* HAVE_LIBJSON  */
+	return lwgeom;
+#endif /* HAVE_LIBJSON } */
 }
 
 
