@@ -14,9 +14,9 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "liblwgeom_internal.h"
-
+#include "../postgis_config.h"
 /*#define POSTGIS_DEBUG_LEVEL 4*/
+#include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 
 int
@@ -56,12 +56,11 @@ ptarray_construct(char hasz, char hasm, uint32_t npoints)
 POINTARRAY*
 ptarray_construct_empty(char hasz, char hasm, uint32_t maxpoints)
 {
-	uint8_t dims = gflags(hasz, hasm, 0);
 	POINTARRAY *pa = lwalloc(sizeof(POINTARRAY));
 	pa->serialized_pointlist = NULL;
 	
 	/* Set our dimsionality info on the bitmap */
-	pa->flags = dims;
+	pa->flags = gflags(hasz, hasm, 0);
 	
 	/* We will be allocating a bit of room */
 	pa->npoints = 0;
@@ -110,7 +109,10 @@ ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, int where)
 
 	/* Error out if we have a bad situation */
 	if ( pa->npoints > pa->maxpoints )
+	{
 		lwerror("npoints (%d) is greated than maxpoints (%d)", pa->npoints, pa->maxpoints);
+		return LW_FAILURE;
+	}
 	
 	/* Check if we have enough storage, add more if necessary */
 	if( pa->npoints == pa->maxpoints )
@@ -369,6 +371,30 @@ ptarray_flip_coordinates(POINTARRAY *pa)
 	return pa;
 }
 
+void
+ptarray_swap_ordinates(POINTARRAY *pa, LWORD o1, LWORD o2)
+{
+	int i;
+	double d, *dp1, *dp2;
+	POINT4D p;
+
+#if PARANOIA_LEVEL > 0
+  assert(o1 < 4);
+  assert(o2 < 4);
+#endif
+
+  dp1 = ((double*)&p)+(unsigned)o1;
+  dp2 = ((double*)&p)+(unsigned)o2;
+	for (i=0 ; i < pa->npoints ; i++)
+	{
+		getPoint4d_p(pa, i, &p);
+		d = *dp2;
+		*dp2 = *dp1;
+		*dp1 = d;
+		ptarray_set_point4d(pa, i, &p);
+	}
+}
+
 
 /**
  * @brief Returns a modified #POINTARRAY so that no segment is
@@ -434,6 +460,8 @@ ptarray_segmentize2d(const POINTARRAY *ipa, double dist)
 			p1 = p2;
 			ipoff++;
 		}
+
+		LW_ON_INTERRUPT(ptarray_free(opa); return NULL);
 	}
 
 	return opa;
@@ -464,19 +492,6 @@ ptarray_same(const POINTARRAY *pa1, const POINTARRAY *pa2)
 	return LW_TRUE;
 }
 
-
-
-/**
- * @brief Add a point in a pointarray.
- *
- * @param pa the source POINTARRAY
- * @param p the point to add
- * @param pdims number of ordinates in p (2..4)
- * @param where to insert the point. 0 prepends, pa->npoints appends
- *
- * @returns a newly constructed POINTARRAY using a newly allocated buffer
- *          for the actual points, or NULL on error.
- */
 POINTARRAY *
 ptarray_addPoint(const POINTARRAY *pa, uint8_t *p, size_t pdims, uint32_t where)
 {
@@ -530,12 +545,6 @@ ptarray_addPoint(const POINTARRAY *pa, uint8_t *p, size_t pdims, uint32_t where)
 	return ret;
 }
 
-
-/**
- * @brief Remove a point from a pointarray.
- * 	@param which -  is the offset (starting at 0)
- * @return #POINTARRAY is newly allocated
- */
 POINTARRAY *
 ptarray_removePoint(POINTARRAY *pa, uint32_t which)
 {
@@ -577,13 +586,6 @@ ptarray_removePoint(POINTARRAY *pa, uint32_t which)
 	return ret;
 }
 
-
-/**
- * @brief Merge two given POINTARRAY and returns a pointer
- * on the new aggregate one.
- * Warning: this function free the two inputs POINTARRAY
- * @return #POINTARRAY is newly allocated
- */
 POINTARRAY *
 ptarray_merge(POINTARRAY *pa1, POINTARRAY *pa2)
 {
@@ -605,8 +607,8 @@ ptarray_merge(POINTARRAY *pa1, POINTARRAY *pa2)
 	                getPoint_internal(pa2, 0),
 	                ptsize*(pa2->npoints));
 
-	lwfree(pa1);
-	lwfree(pa2);
+	ptarray_free(pa1);
+	ptarray_free(pa2);
 
 	return pa;
 }
@@ -1278,9 +1280,9 @@ ptarray_locate_point(const POINTARRAY *pa, const POINT4D *p4d, double *mindistou
 	p.y = p4d->y;
 	
 	if ( ! proj4d ) proj4d = &projtmp;
-
+	
 	start = getPoint2d_cp(pa, 0);
-
+	
 	/* If the pointarray has only one point, the nearest point is */
 	/* just that point */
 	if ( pa->npoints == 1 )
@@ -1290,7 +1292,7 @@ ptarray_locate_point(const POINTARRAY *pa, const POINT4D *p4d, double *mindistou
 			*mindistout = distance2d_pt_pt(&p, start);
 		return 0.0;
 	}
-
+	
 	/* Loop through pointarray looking for nearest segment */
 	for (t=1; t<pa->npoints; t++)
 	{
@@ -1398,20 +1400,24 @@ ptarray_longitude_shift(POINTARRAY *pa)
  *
  */
 POINTARRAY *
-ptarray_remove_repeated_points(POINTARRAY *in)
+ptarray_remove_repeated_points_minpoints(POINTARRAY *in, double tolerance, int minpoints)
 {
 	POINTARRAY* out;
 	size_t ptsize;
 	size_t ipn, opn;
+	const POINT2D *last_point, *this_point;
+	double tolsq = tolerance * tolerance;
 
-	LWDEBUG(3, "ptarray_remove_repeated_points called.");
+	if ( minpoints < 1 ) minpoints = 1;
+
+	LWDEBUGF(3, "%s called", __func__);
 
 	/* Single or zero point arrays can't have duplicates */
 	if ( in->npoints < 3 ) return ptarray_clone_deep(in);
 
 	ptsize = ptarray_point_size(in);
 
-	LWDEBUGF(3, "ptsize: %d", ptsize);
+	LWDEBUGF(3, " ptsize: %d", ptsize);
 
 	/* Allocate enough space for all points */
 	out = ptarray_construct(FLAGS_GET_Z(in->flags),
@@ -1421,18 +1427,20 @@ ptarray_remove_repeated_points(POINTARRAY *in)
 
 	opn=1;
 	memcpy(getPoint_internal(out, 0), getPoint_internal(in, 0), ptsize);
+	last_point = getPoint2d_cp(in, 0);
 	LWDEBUGF(3, " first point copied, out points: %d", opn);
-	for (ipn=1; ipn<in->npoints; ++ipn)
+	for ( ipn = 1; ipn < in->npoints; ++ipn)
 	{
-		if ( (ipn==in->npoints-1 && opn==1) || memcmp(getPoint_internal(in, ipn-1),
-		        getPoint_internal(in, ipn), ptsize) )
+		this_point = getPoint2d_cp(in, ipn);
+		if ( (ipn >= in->npoints-minpoints+1 && opn < minpoints) || 
+		     (tolerance == 0 && memcmp(getPoint_internal(in, ipn-1), getPoint_internal(in, ipn), ptsize) != 0) ||
+		     (tolerance > 0.0 && distance2d_sqr_pt_pt(last_point, this_point) > tolsq) )
 		{
 			/* The point is different from the previous,
 			 * we add it to output */
-			memcpy(getPoint_internal(out, opn++),
-			       getPoint_internal(in, ipn), ptsize);
-			LWDEBUGF(3, " Point %d differs from point %d. Out points: %d",
-			         ipn, ipn-1, opn);
+			memcpy(getPoint_internal(out, opn++), getPoint_internal(in, ipn), ptsize);
+			last_point = this_point;
+			LWDEBUGF(3, " Point %d differs from point %d. Out points: %d", ipn, ipn-1, opn);
 		}
 	}
 
@@ -1442,6 +1450,12 @@ ptarray_remove_repeated_points(POINTARRAY *in)
 	return out;
 }
 
+POINTARRAY *
+ptarray_remove_repeated_points(POINTARRAY *in, double tolerance)
+{
+	return ptarray_remove_repeated_points_minpoints(in, tolerance, 2);
+}
+
 static void
 ptarray_dp_findsplit(POINTARRAY *pts, int p1, int p2, int *split, double *dist)
 {
@@ -1638,8 +1652,10 @@ ptarray_length(const POINTARRAY *pts)
 
 /*
  * Get a pointer to nth point of a POINTARRAY.
- * You cannot safely cast this to a real POINT, due to memory alignment
- * constraints. Use getPoint*_p for that.
+ *
+ * Casting to returned pointer to POINT2D* should be safe,
+ * as gserialized format always keeps the POINTARRAY pointer
+ * aligned to double boundary.
  */
 uint8_t *
 getPoint_internal(const POINTARRAY *pa, int n)
@@ -1736,8 +1752,100 @@ ptarray_affine(POINTARRAY *pa, const AFFINE *a)
 
 }
 
+/**
+ * Scale a pointarray.
+ */
+void
+ptarray_scale(POINTARRAY *pa, const POINT4D *fact)
+{
+  int i;
+  POINT4D p4d;
+
+  LWDEBUG(3, "ptarray_scale start");
+
+  for (i=0; i<pa->npoints; ++i)
+  {
+    getPoint4d_p(pa, i, &p4d);
+    p4d.x *= fact->x;
+    p4d.y *= fact->y;
+    p4d.z *= fact->z;
+    p4d.m *= fact->m;
+    ptarray_set_point4d(pa, i, &p4d);
+  }
+
+  LWDEBUG(3, "ptarray_scale end");
+
+}
+
 int
 ptarray_startpoint(const POINTARRAY* pa, POINT4D* pt)
 {
 	return getPoint4d_p(pa, 0, pt);
 }
+
+
+
+
+/*
+ * Stick an array of points to the given gridspec.
+ * Return "gridded" points in *outpts and their number in *outptsn.
+ *
+ * Two consecutive points falling on the same grid cell are collapsed
+ * into one single point.
+ *
+ */
+POINTARRAY *
+ptarray_grid(const POINTARRAY *pa, const gridspec *grid)
+{
+	POINT4D pt;
+	int ipn; /* input point numbers */
+	POINTARRAY *dpa;
+
+	LWDEBUGF(2, "ptarray_grid called on %p", pa);
+
+	dpa = ptarray_construct_empty(FLAGS_GET_Z(pa->flags),FLAGS_GET_M(pa->flags), pa->npoints);
+
+	for (ipn=0; ipn<pa->npoints; ++ipn)
+	{
+
+		getPoint4d_p(pa, ipn, &pt);
+
+		if ( grid->xsize )
+			pt.x = rint((pt.x - grid->ipx)/grid->xsize) *
+			         grid->xsize + grid->ipx;
+
+		if ( grid->ysize )
+			pt.y = rint((pt.y - grid->ipy)/grid->ysize) *
+			         grid->ysize + grid->ipy;
+
+		if ( FLAGS_GET_Z(pa->flags) && grid->zsize )
+			pt.z = rint((pt.z - grid->ipz)/grid->zsize) *
+			         grid->zsize + grid->ipz;
+
+		if ( FLAGS_GET_M(pa->flags) && grid->msize )
+			pt.m = rint((pt.m - grid->ipm)/grid->msize) *
+			         grid->msize + grid->ipm;
+
+		ptarray_append_point(dpa, &pt, LW_FALSE);
+
+	}
+
+	return dpa;
+}
+
+int 
+ptarray_npoints_in_rect(const POINTARRAY *pa, const GBOX *gbox)
+{
+	const POINT2D *pt;
+	int n = 0;
+	int i;
+	for ( i = 0; i < pa->npoints; i++ )
+	{
+		pt = getPoint2d_cp(pa, i);
+		if ( gbox_contains_point2d(gbox, pt) )
+			n++;
+	}
+	return n;
+}
+
+
