@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: geography_inout.c 4535 2009-09-28 18:16:21Z colivier $
  *
  * PostGIS - Spatial Types for PostgreSQL
  *
@@ -27,8 +26,17 @@
 #include "geography_measurement_trees.h" /* For circ_tree caching */
 #include "lwgeom_transform.h" /* For SRID functions */
 
+#if PROJ_GEODESIC
+/* round to 10 nm precision */
+#define INVMINDIST 1.0e8
+#else
+/* round to 100 nm precision */
+#define INVMINDIST 1.0e9
+#endif
+
 Datum geography_distance(PG_FUNCTION_ARGS);
 Datum geography_distance_uncached(PG_FUNCTION_ARGS);
+Datum geography_distance_knn(PG_FUNCTION_ARGS);
 Datum geography_distance_tree(PG_FUNCTION_ARGS);
 Datum geography_dwithin(PG_FUNCTION_ARGS);
 Datum geography_dwithin_uncached(PG_FUNCTION_ARGS);
@@ -43,6 +51,66 @@ Datum geography_project(PG_FUNCTION_ARGS);
 Datum geography_azimuth(PG_FUNCTION_ARGS);
 Datum geography_segmentize(PG_FUNCTION_ARGS);
 
+
+PG_FUNCTION_INFO_V1(geography_distance_knn);
+Datum geography_distance_knn(PG_FUNCTION_ARGS)
+{
+	LWGEOM *lwgeom1 = NULL;
+	LWGEOM *lwgeom2 = NULL;
+	GSERIALIZED *g1 = NULL;
+	GSERIALIZED *g2 = NULL;
+	double distance;
+	double tolerance = FP_TOLERANCE;
+	bool use_spheroid = false; /* must use sphere, can't get index to harmonize with spheroid */ 
+	SPHEROID s;
+
+	/* Get our geometry objects loaded into memory. */
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
+
+	/* Initialize spheroid */
+	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
+
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
+	
+	/* Set to sphere if requested */
+	if ( ! use_spheroid )
+		s.a = s.b = s.radius;
+
+	lwgeom1 = lwgeom_from_gserialized(g1);
+	lwgeom2 = lwgeom_from_gserialized(g2);
+
+	/* Return NULL on empty arguments. */
+	if ( lwgeom_is_empty(lwgeom1) || lwgeom_is_empty(lwgeom2) )
+	{
+		PG_FREE_IF_COPY(g1, 0);
+		PG_FREE_IF_COPY(g2, 1);
+		PG_RETURN_NULL();
+	}
+
+	/* Make sure we have boxes attached */
+	lwgeom_add_bbox_deep(lwgeom1, NULL);
+	lwgeom_add_bbox_deep(lwgeom2, NULL);
+	
+	distance = lwgeom_distance_spheroid(lwgeom1, lwgeom2, &s, tolerance);
+
+	POSTGIS_DEBUGF(2, "[GIST] '%s' got distance %g", __func__, distance);
+
+	/* Clean up */
+	lwgeom_free(lwgeom1);
+	lwgeom_free(lwgeom2);
+	PG_FREE_IF_COPY(g1, 0);
+	PG_FREE_IF_COPY(g2, 1);
+
+	/* Something went wrong, negative return... should already be eloged, return NULL */
+	if ( distance < 0.0 )
+	{
+		PG_RETURN_NULL();
+	}
+
+	PG_RETURN_FLOAT8(distance);
+}
+
 /*
 ** geography_distance_uncached(GSERIALIZED *g1, GSERIALIZED *g2, double tolerance, boolean use_spheroid)
 ** returns double distance in meters
@@ -55,19 +123,23 @@ Datum geography_distance_uncached(PG_FUNCTION_ARGS)
 	GSERIALIZED *g1 = NULL;
 	GSERIALIZED *g2 = NULL;
 	double distance;
-	/* double tolerance; */
-	bool use_spheroid;
+	double tolerance = FP_TOLERANCE;
+	bool use_spheroid = true; 
 	SPHEROID s;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Read our tolerance value. */
-	/* tolerance = PG_GETARG_FLOAT8(2); */
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		tolerance = PG_GETARG_FLOAT8(2);
     
 	/* Read our calculation type. */
-	use_spheroid = PG_GETARG_BOOL(3);
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		use_spheroid = PG_GETARG_BOOL(3);
+
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
 
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
@@ -91,7 +163,9 @@ Datum geography_distance_uncached(PG_FUNCTION_ARGS)
 	lwgeom_add_bbox_deep(lwgeom1, NULL);
 	lwgeom_add_bbox_deep(lwgeom2, NULL);
 	
-	distance = lwgeom_distance_spheroid(lwgeom1, lwgeom2, &s, FP_TOLERANCE);
+	distance = lwgeom_distance_spheroid(lwgeom1, lwgeom2, &s, tolerance);
+
+	POSTGIS_DEBUGF(2, "[GIST] '%s' got distance %g", __func__, distance);
 
 	/* Clean up */
 	lwgeom_free(lwgeom1);
@@ -119,20 +193,24 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 	GSERIALIZED* g1 = NULL;
 	GSERIALIZED* g2 = NULL;
 	double distance;
-	double tolerance;
-	bool use_spheroid;
+	double tolerance = 0.0;
+	bool use_spheroid = true;
 	SPHEROID s;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Read our tolerance value. */
-	tolerance = PG_GETARG_FLOAT8(2);
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		tolerance = PG_GETARG_FLOAT8(2);
 
 	/* Read our calculation type. */
-	use_spheroid = PG_GETARG_BOOL(3);
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		use_spheroid = PG_GETARG_BOOL(3);
 
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
+	
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 	
@@ -162,6 +240,9 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 	PG_FREE_IF_COPY(g1, 0);
 	PG_FREE_IF_COPY(g2, 1);
 
+	/* Knock off any funny business at the nanometer level, ticket #2168 */
+	distance = round(distance * INVMINDIST) / INVMINDIST;
+
 	/* Something went wrong, negative return... should already be eloged, return NULL */
 	if ( distance < 0.0 )
 	{
@@ -169,9 +250,6 @@ Datum geography_distance(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-    /* Knock off any funny business at the micrometer level, ticket #2168 */
-    distance = round(distance * 10e8) / 10e8;
-
 	PG_RETURN_FLOAT8(distance);
 }
 
@@ -185,21 +263,25 @@ Datum geography_dwithin(PG_FUNCTION_ARGS)
 {
 	GSERIALIZED *g1 = NULL;
 	GSERIALIZED *g2 = NULL;
-	double tolerance;
+	double tolerance = 0.0;
 	double distance;
-	bool use_spheroid;
+	bool use_spheroid = true;
 	SPHEROID s;
 	int dwithin = LW_FALSE;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Read our tolerance value. */
-	tolerance = PG_GETARG_FLOAT8(2);
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		tolerance = PG_GETARG_FLOAT8(2);
 
 	/* Read our calculation type. */
-	use_spheroid = PG_GETARG_BOOL(3);
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		use_spheroid = PG_GETARG_BOOL(3);
+
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
 
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
@@ -247,14 +329,14 @@ Datum geography_distance_tree(PG_FUNCTION_ARGS)
 {
 	GSERIALIZED *g1 = NULL;
 	GSERIALIZED *g2 = NULL;
-	double tolerance;
+	double tolerance = 0.0;
 	double distance;
-	bool use_spheroid;
+	bool use_spheroid = true;
 	SPHEROID s;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Return FALSE on empty arguments. */
 	if ( gserialized_is_empty(g1) || gserialized_is_empty(g2) )
@@ -265,11 +347,15 @@ Datum geography_distance_tree(PG_FUNCTION_ARGS)
 	}
 
 	/* Read our tolerance value. */
-	tolerance = PG_GETARG_FLOAT8(2);
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		tolerance = PG_GETARG_FLOAT8(2);
 
 	/* Read our calculation type. */
-	use_spheroid = PG_GETARG_BOOL(3);
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		use_spheroid = PG_GETARG_BOOL(3);
 
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
+	
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
 
@@ -299,20 +385,24 @@ Datum geography_dwithin_uncached(PG_FUNCTION_ARGS)
 	LWGEOM *lwgeom2 = NULL;
 	GSERIALIZED *g1 = NULL;
 	GSERIALIZED *g2 = NULL;
-	double tolerance;
+	double tolerance = 0.0;
 	double distance;
-	bool use_spheroid;
+	bool use_spheroid = true;
 	SPHEROID s;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	/* Read our tolerance value. */
-	tolerance = PG_GETARG_FLOAT8(2);
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		tolerance = PG_GETARG_FLOAT8(2);
 
 	/* Read our calculation type. */
-	use_spheroid = PG_GETARG_BOOL(3);
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		use_spheroid = PG_GETARG_BOOL(3);
+
+	error_if_srid_mismatch(gserialized_get_srid(g1), gserialized_get_srid(g2));
 
 	/* Initialize spheroid */
 	spheroid_init_from_srid(fcinfo, gserialized_get_srid(g1), &s);
@@ -365,7 +455,7 @@ Datum geography_expand(PG_FUNCTION_ARGS)
 	double distance;
 
 	/* Get a wholly-owned pointer to the geography */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM_COPY(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P_COPY(0);
 
 	/* Read our distance value and normalize to unit-sphere. */
 	distance = PG_GETARG_FLOAT8(1) / WGS84_RADIUS;
@@ -402,7 +492,7 @@ Datum geography_area(PG_FUNCTION_ARGS)
 	SPHEROID s;
 
 	/* Get our geometry object loaded into memory. */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 
 	/* Read our calculation type */
 	use_spheroid = PG_GETARG_BOOL(1);
@@ -424,6 +514,7 @@ Datum geography_area(PG_FUNCTION_ARGS)
 	else
 		lwgeom_calculate_gbox_geodetic(lwgeom, &gbox);
 
+#if ! PROJ_GEODESIC
 	/* Test for cases that are currently not handled by spheroid code */
 	if ( use_spheroid )
 	{
@@ -434,6 +525,7 @@ Datum geography_area(PG_FUNCTION_ARGS)
 		if ( gbox.zmax > 0.0 && gbox.zmin < 0.0 )
 			use_spheroid = LW_FALSE;
 	}
+#endif /* if ! PROJ_GEODESIC */
 
 	/* User requests spherical calculation, turn our spheroid into a sphere */
 	if ( ! use_spheroid )
@@ -471,17 +563,17 @@ Datum geography_perimeter(PG_FUNCTION_ARGS)
 	double length;
 	bool use_spheroid = LW_TRUE;
 	SPHEROID s;
-    int type;
+	int type;
 
 	/* Get our geometry object loaded into memory. */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	
 	/* Only return for area features. */
-    type = gserialized_get_type(g);
-    if ( ! (type == POLYGONTYPE || type == MULTIPOLYGONTYPE || type == COLLECTIONTYPE) )
-    {
+	type = gserialized_get_type(g);
+	if ( ! (type == POLYGONTYPE || type == MULTIPOLYGONTYPE || type == COLLECTIONTYPE) )
+	{
 		PG_RETURN_FLOAT8(0.0);
-    }
+	}
 	
 	lwgeom = lwgeom_from_gserialized(g);
 
@@ -533,7 +625,7 @@ Datum geography_length(PG_FUNCTION_ARGS)
 	SPHEROID s;
 
 	/* Get our geometry object loaded into memory. */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	lwgeom = lwgeom_from_gserialized(g);
 
 	/* EMPTY things have no length */
@@ -585,7 +677,7 @@ Datum geography_point_outside(PG_FUNCTION_ARGS)
 	POINT2D pt;
 
 	/* Get our geometry object loaded into memory. */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	
 	/* We need the bounding box to get an outside point for area algorithm */
 	if ( gserialized_get_gbox_p(g, &gbox) == LW_FAILURE )
@@ -627,8 +719,8 @@ Datum geography_covers(PG_FUNCTION_ARGS)
 	int result = LW_FALSE;
 
 	/* Get our geometry objects loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 
 	type1 = gserialized_get_type(g1);
 	type2 = gserialized_get_type(g2);
@@ -645,6 +737,8 @@ Datum geography_covers(PG_FUNCTION_ARGS)
 	lwgeom1 = lwgeom_from_gserialized(g1);
 	lwgeom2 = lwgeom_from_gserialized(g2);
 
+	error_if_srid_mismatch(lwgeom1->srid, lwgeom2->srid);
+
 	/* EMPTY never intersects with another geometry */
 	if ( lwgeom_is_empty(lwgeom1) || lwgeom_is_empty(lwgeom2) )
 	{
@@ -831,24 +925,25 @@ Datum geography_project(PG_FUNCTION_ARGS)
 	LWPOINT *lwp_projected;
 	GSERIALIZED *g = NULL;
 	GSERIALIZED *g_out = NULL;
-	double azimuth, distance;
+	double azimuth = 0.0;
+	double distance;
 	SPHEROID s;
 	uint32_t type;
 
 	/* Return NULL on NULL distance or geography */
-	if ( PG_ARGISNULL(0) || PG_ARGISNULL(1) )
+	if ( PG_NARGS() < 2 || PG_ARGISNULL(0) || PG_ARGISNULL(1) )
 		PG_RETURN_NULL();
 	
 	/* Get our geometry object loaded into memory. */
-	g = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	g = PG_GETARG_GSERIALIZED_P(0);
 	
 	/* Only return for points. */
-    type = gserialized_get_type(g);
-    if ( type != POINTTYPE )
-    {
+	type = gserialized_get_type(g);
+	if ( type != POINTTYPE )
+	{
 		elog(ERROR, "ST_Project(geography) is only valid for point inputs");
 		PG_RETURN_NULL();
-    }
+	}
 
 	distance = PG_GETARG_FLOAT8(1); /* Distance in Meters */
 	lwgeom = lwgeom_from_gserialized(g);
@@ -861,9 +956,7 @@ Datum geography_project(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 	
-	if ( PG_ARGISNULL(2) )
-		azimuth = 0.0;
-	else
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
 		azimuth = PG_GETARG_FLOAT8(2); /* Azimuth in Radians */
 
 	/* Initialize spheroid */
@@ -912,17 +1005,17 @@ Datum geography_azimuth(PG_FUNCTION_ARGS)
 	uint32_t type1, type2;
 
 	/* Get our geometry object loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	g2 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	g2 = PG_GETARG_GSERIALIZED_P(1);
 	
 	/* Only return for points. */
-    type1 = gserialized_get_type(g1);
-    type2 = gserialized_get_type(g2);
-    if ( type1 != POINTTYPE || type2 != POINTTYPE )
-    {
+	type1 = gserialized_get_type(g1);
+	type2 = gserialized_get_type(g2);
+	if ( type1 != POINTTYPE || type2 != POINTTYPE )
+	{
 		elog(ERROR, "ST_Azimuth(geography, geography) is only valid for point inputs");
 		PG_RETURN_NULL();
-    }
+	}
 	
 	lwgeom1 = lwgeom_from_gserialized(g1);
 	lwgeom2 = lwgeom_from_gserialized(g2);
@@ -975,14 +1068,14 @@ Datum geography_segmentize(PG_FUNCTION_ARGS)
 	uint32_t type1;
 
 	/* Get our geometry object loaded into memory. */
-	g1 = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-    type1 = gserialized_get_type(g1);
+	g1 = PG_GETARG_GSERIALIZED_P(0);
+	type1 = gserialized_get_type(g1);
 
 	/* Convert max_seg_length from metric units to radians */
 	max_seg_length = PG_GETARG_FLOAT8(1) / WGS84_RADIUS;
 
 	/* We can't densify points or points, reflect them back */
-    if ( type1 == POINTTYPE || type1 == MULTIPOINTTYPE || gserialized_is_empty(g1) )
+	if ( type1 == POINTTYPE || type1 == MULTIPOINTTYPE || gserialized_is_empty(g1) )
 		PG_RETURN_POINTER(g1);
 
 	/* Deserialize */
