@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: geography_inout.c 7248 2011-05-25 18:42:16Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
@@ -29,7 +28,8 @@
 #include "lwgeom_pg.h"       /* For debugging macros. */
 #include "geography.h"	     /* For utility functions. */
 #include "lwgeom_export.h"   /* For export functions. */
-#include "lwgeom_transform.h" /* for srid_is_latlon */ 
+#include "lwgeom_transform.h" /* for srid_is_latlon */
+
 
 Datum geography_typmod_in(PG_FUNCTION_ARGS);
 Datum geometry_typmod_in(PG_FUNCTION_ARGS);
@@ -96,11 +96,12 @@ Datum postgis_typmod_out(PG_FUNCTION_ARGS)
 
 }
 
+
 /**
 * Check the consistency of the metadata we want to enforce in the typmod:
 * srid, type and dimensionality. If things are inconsistent, shut down the query.
 */
-GSERIALIZED * postgis_valid_typmod(GSERIALIZED *gser, int32_t typmod)
+GSERIALIZED* postgis_valid_typmod(GSERIALIZED *gser, int32_t typmod)
 {
 	int32 geom_srid = gserialized_get_srid(gser);
 	int32 geom_type = gserialized_get_type(gser);
@@ -132,8 +133,8 @@ GSERIALIZED * postgis_valid_typmod(GSERIALIZED *gser, int32_t typmod)
 	     gserialized_is_empty(gser) )
 	{
 		LWPOINT *empty_point = lwpoint_construct_empty(geom_srid, geom_z, geom_m);
-		pfree(gser);
 		geom_type = POINTTYPE;
+		pfree(gser);
 		if ( gserialized_is_geodetic(gser) )
 			gser = geography_serialize(lwpoint_as_lwgeom(empty_point));
 		else
@@ -264,7 +265,7 @@ static uint32 gserialized_typmod_in(ArrayType *arr, int is_geography)
 			POSTGIS_DEBUGF(3, "srid: %d", srid);
 			if ( srid != SRID_UNKNOWN )
 			{
-				TYPMOD_SET_SRID(typmod, srid); 
+				TYPMOD_SET_SRID(typmod, srid);
 			}
 		}
 	}
@@ -284,10 +285,10 @@ Datum geography_typmod_in(PG_FUNCTION_ARGS)
 {
 	ArrayType *arr = (ArrayType *) DatumGetPointer(PG_GETARG_DATUM(0));
 	uint32 typmod = gserialized_typmod_in(arr, LW_TRUE);
-	int srid = TYPMOD_GET_SRID(typmod); 
-	/* Check the SRID is legal (geographic coordinates) */ 
-	srid_is_latlong(fcinfo, srid); 
-
+	int srid = TYPMOD_GET_SRID(typmod);
+	/* Check the SRID is legal (geographic coordinates) */
+	srid_is_latlong(fcinfo, srid);
+	
 	PG_RETURN_INT32(typmod);
 }
 
@@ -312,7 +313,7 @@ Datum geometry_typmod_in(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(geography_enforce_typmod);
 Datum geography_enforce_typmod(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *arg = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *arg = PG_GETARG_GSERIALIZED_P(0);
 	int32 typmod = PG_GETARG_INT32(1);
 	/* We don't need to have different behavior based on explicitness. */
 	/* bool isExplicit = PG_GETARG_BOOL(2); */
@@ -331,7 +332,7 @@ Datum geography_enforce_typmod(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(geometry_enforce_typmod);
 Datum geometry_enforce_typmod(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *arg = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *arg = PG_GETARG_GSERIALIZED_P(0);
 	int32 typmod = PG_GETARG_INT32(1);
 	/* We don't need to have different behavior based on explicitness. */
 	/* bool isExplicit = PG_GETARG_BOOL(2); */
