@@ -28,7 +28,7 @@ Datum ST_AddMeasure(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_AddMeasure);
 Datum ST_AddMeasure(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *gin = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *gin = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *gout;
 	double start_measure = PG_GETARG_FLOAT8(1);
 	double end_measure = PG_GETARG_FLOAT8(2);
@@ -38,7 +38,7 @@ Datum ST_AddMeasure(PG_FUNCTION_ARGS)
 	/* Raise an error if input is not a linestring or multilinestring */
 	if ( type != LINETYPE && type != MULTILINETYPE )
 	{
-		lwerror("Only LINESTRING and MULTILINESTRING are supported");
+		lwpgerror("Only LINESTRING and MULTILINESTRING are supported");
 		PG_RETURN_NULL();
 	}
 
@@ -68,7 +68,7 @@ Datum ST_LocateAlong(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_LocateAlong);
 Datum ST_LocateAlong(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *gin = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *gin = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *gout;
 	LWGEOM *lwin = NULL, *lwout = NULL;
 	double measure = PG_GETARG_FLOAT8(1);
@@ -96,7 +96,7 @@ Datum ST_LocateBetween(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_LocateBetween);
 Datum ST_LocateBetween(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom_in = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom_in = PG_GETARG_GSERIALIZED_P(0);
 	double from = PG_GETARG_FLOAT8(1);
 	double to = PG_GETARG_FLOAT8(2);
 	double offset = PG_GETARG_FLOAT8(3);
@@ -137,7 +137,7 @@ Datum ST_LocateBetweenElevations(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_LocateBetweenElevations);
 Datum ST_LocateBetweenElevations(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom_in = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom_in = PG_GETARG_GSERIALIZED_P(0);
 	double from = PG_GETARG_FLOAT8(1);
 	double to = PG_GETARG_FLOAT8(2);
 	LWCOLLECTION *geom_out = NULL;
@@ -170,8 +170,8 @@ Datum ST_InterpolatePoint(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_InterpolatePoint);
 Datum ST_InterpolatePoint(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *gser_line = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	GSERIALIZED *gser_point = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	GSERIALIZED *gser_line = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *gser_point = PG_GETARG_GSERIALIZED_P(1);
 	LWGEOM *lwline;
 	LWPOINT *lwpoint;
 
@@ -185,11 +185,9 @@ Datum ST_InterpolatePoint(PG_FUNCTION_ARGS)
 		elog(ERROR,"ST_InterpolatePoint: 2st argument isn't a point");
 		PG_RETURN_NULL();
 	}
-	if ( gserialized_get_srid(gser_line) != gserialized_get_srid(gser_point) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+
+	error_if_srid_mismatch(gserialized_get_srid(gser_line), gserialized_get_srid(gser_point));
+
 	if ( ! gserialized_has_m(gser_line) )
 	{
 		elog(ERROR,"ST_InterpolatePoint only accepts geometries that have an M dimension");
@@ -207,8 +205,8 @@ Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(LWGEOM_line_locate_point);
 Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom1 = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	GSERIALIZED *geom2 = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	GSERIALIZED *geom1 = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *geom2 = PG_GETARG_GSERIALIZED_P(1);
 	LWLINE *lwline;
 	LWPOINT *lwpoint;
 	POINTARRAY *pa;
@@ -225,11 +223,8 @@ Datum LWGEOM_line_locate_point(PG_FUNCTION_ARGS)
 		elog(ERROR,"line_locate_point: 2st arg isnt a point");
 		PG_RETURN_NULL();
 	}
-	if ( gserialized_get_srid(geom1) != gserialized_get_srid(geom2) )
-	{
-		elog(ERROR, "Operation on two geometries with different SRIDs");
-		PG_RETURN_NULL();
-	}
+
+	error_if_srid_mismatch(gserialized_get_srid(geom1), gserialized_get_srid(geom2));
 
 	lwline = lwgeom_as_lwline(lwgeom_from_gserialized(geom1));
 	lwpoint = lwgeom_as_lwpoint(lwgeom_from_gserialized(geom2));
@@ -442,7 +437,7 @@ ptarray_locate_between_m(POINTARRAY *ipa, double m0, double m1)
 	 */
 	ret.ptarrays=lwalloc(sizeof(POINTARRAY *)*ipa->npoints-1);
 
-	POSTGIS_DEBUGF(2, "ptarray_locate...: called for pointarray %x, m0:%g, m1:%g",
+	POSTGIS_DEBUGF(2, "ptarray_locate...: called for pointarray %p, m0:%g, m1:%g",
 	         ipa, m0, m1);
 
 
@@ -464,7 +459,7 @@ ptarray_locate_between_m(POINTARRAY *ipa, double m0, double m1)
 		/* segment completely outside, nothing to do */
 		if (! clipval ) continue;
 
-		POSTGIS_DEBUGF(3, " clipped to: [ %g %g %g %g - %g %g %g %g ]   clipval: %x", p1.x, p1.y, p1.z, p1.m,
+		POSTGIS_DEBUGF(3, " clipped to: [ %g %g %g %g - %g %g %g %g ]   clipval: %d", p1.x, p1.y, p1.z, p1.m,
 		         p2.x, p2.y, p2.z, p2.m, clipval);
 
 		/* If no points have been accumulated so far, then if clipval != 0 the first point must be the
@@ -486,7 +481,7 @@ ptarray_locate_between_m(POINTARRAY *ipa, double m0, double m1)
 		 */
 		if ( clipval & 0x0100 || i == ipa->npoints-1 )
 		{
-			POSTGIS_DEBUGF(3, " closing pointarray %x with %d points", dpa, dpa->npoints);
+			POSTGIS_DEBUGF(3, " closing pointarray %p with %d points", dpa, dpa->npoints);
 
 			ret.ptarrays[ret.nptarrays++] = dpa;
 			dpa = NULL;
@@ -497,7 +492,7 @@ ptarray_locate_between_m(POINTARRAY *ipa, double m0, double m1)
 	 * if dpa!=NULL it means we didn't close it yet.
 	 * this should never happen.
 	 */
-	if ( dpa != NULL ) lwerror("Something wrong with algorithm");
+	if ( dpa != NULL ) lwpgerror("Something wrong with algorithm");
 
 	return ret;
 }
@@ -512,7 +507,7 @@ lwpoint_locate_between_m(LWPOINT *lwpoint, double m0, double m1)
 {
 	POINT3DM p3dm;
 
-	POSTGIS_DEBUGF(2, "lwpoint_locate_between called for lwpoint %x", lwpoint);
+	POSTGIS_DEBUGF(2, "lwpoint_locate_between called for lwpoint %p", lwpoint);
 
 	lwpoint_getPoint3dm_p(lwpoint, &p3dm);
 	if ( p3dm.m >= m0 && p3dm.m <= m1)
@@ -553,7 +548,7 @@ lwline_locate_between_m(LWLINE *lwline_in, double m0, double m1)
 	const int lineflag=0x10;
 	POINTARRAYSET paset=ptarray_locate_between_m(ipa, m0, m1);
 
-	POSTGIS_DEBUGF(2, "lwline_locate_between called for lwline %x", lwline_in);
+	POSTGIS_DEBUGF(2, "lwline_locate_between called for lwline %p", lwline_in);
 
 	POSTGIS_DEBUGF(3, " ptarray_locate... returned %d pointarrays",
 	         paset.nptarrays);
@@ -592,7 +587,7 @@ lwline_locate_between_m(LWLINE *lwline_in, double m0, double m1)
 		/* This is a bug */
 		else
 		{
-			lwerror("ptarray_locate_between_m returned a POINARRAY set containing POINTARRAY with 0 points");
+			lwpgerror("ptarray_locate_between_m returned a POINARRAY set containing POINTARRAY with 0 points");
 		}
 
 	}
@@ -624,7 +619,7 @@ lwcollection_locate_between_m(LWCOLLECTION *lwcoll, double m0, double m1)
 	int ngeoms=0;
 	LWGEOM **geoms;
 
-	POSTGIS_DEBUGF(2, "lwcollection_locate_between_m called for lwcoll %x", lwcoll);
+	POSTGIS_DEBUGF(2, "lwcollection_locate_between_m called for lwcoll %p", lwcoll);
 
 	geoms=lwalloc(sizeof(LWGEOM *)*lwcoll->ngeoms);
 	for (i=0; i<lwcoll->ngeoms; i++)
@@ -653,7 +648,7 @@ lwcollection_locate_between_m(LWCOLLECTION *lwcoll, double m0, double m1)
 static LWGEOM *
 lwgeom_locate_between_m(LWGEOM *lwin, double m0, double m1)
 {
-	POSTGIS_DEBUGF(2, "lwgeom_locate_between called for lwgeom %x", lwin);
+	POSTGIS_DEBUGF(2, "lwgeom_locate_between called for lwgeom %p", lwin);
 
 	switch (lwin->type)
 	{
@@ -673,11 +668,11 @@ lwgeom_locate_between_m(LWGEOM *lwin, double m0, double m1)
 		/* Polygon types are not supported */
 	case POLYGONTYPE:
 	case MULTIPOLYGONTYPE:
-		lwerror("Areal geometries are not supported by locate_between_measures");
+		lwpgerror("Areal geometries are not supported by locate_between_measures");
 		return NULL;
 	}
 
-	lwerror("Unkonwn geometry type (%s:%d)", __FILE__, __LINE__);
+	lwpgerror("Unkonwn geometry type (%s:%d)", __FILE__, __LINE__);
 	return NULL;
 }
 
@@ -693,7 +688,7 @@ Datum LWGEOM_locate_between_m(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(LWGEOM_locate_between_m);
 Datum LWGEOM_locate_between_m(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *gin = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *gin = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *gout;
 	double start_measure = PG_GETARG_FLOAT8(1);
 	double end_measure = PG_GETARG_FLOAT8(2);
@@ -702,11 +697,11 @@ Datum LWGEOM_locate_between_m(PG_FUNCTION_ARGS)
 	int hasm = gserialized_has_m(gin);
 	int type;
 
-	elog(NOTICE,"ST_Locate_Between_Measures and ST_Locate_Along_Measure are deprecated. Use ST_LocateAlong and ST_LocateBetween.");	
+	elog(WARNING,"ST_Locate_Between_Measures and ST_Locate_Along_Measure were deprecated in 2.2.0. Please use ST_LocateAlong and ST_LocateBetween");
 
 	if ( end_measure < start_measure )
 	{
-		lwerror("locate_between_m: 2nd arg must be bigger then 1st arg");
+		lwpgerror("locate_between_m: 2nd arg must be bigger then 1st arg");
 		PG_RETURN_NULL();
 	}
 	
@@ -715,7 +710,7 @@ Datum LWGEOM_locate_between_m(PG_FUNCTION_ARGS)
 	 */
 	if ( ! hasm )
 	{
-		lwerror("Geometry argument does not have an 'M' ordinate");
+		lwpgerror("Geometry argument does not have an 'M' ordinate");
 		PG_RETURN_NULL();
 	}
 
@@ -727,7 +722,7 @@ Datum LWGEOM_locate_between_m(PG_FUNCTION_ARGS)
 
 	if ( type == POLYGONTYPE || type == MULTIPOLYGONTYPE || type == COLLECTIONTYPE )
 	{
-		lwerror("Areal or Collection types are not supported");
+		lwpgerror("Areal or Collection types are not supported");
 		PG_RETURN_NULL();
 	}
 
