@@ -1,4 +1,17 @@
 #!/usr/bin/perl
+
+#
+# PostGIS - Spatial Types for PostgreSQL
+# http://postgis.net
+#
+# Copyright (C) 2012-2014 Sandro Santilli <strk@keybit.net>
+# Copyright (C) 2014-2015 Regina Obe <lr@pcorp.us>
+# Copyright (C) 2012-2013 Paul Ramsey <pramsey@cleverelephant.ca>
+#
+# This is free software; you can redistribute and/or modify it under
+# the terms of the GNU General Public Licence. See the COPYING file.
+#
+
 #$| = 1;
 use File::Basename;
 use File::Temp 'tempdir';
@@ -73,17 +86,22 @@ if ( $OPT_UPGRADE_PATH )
 {
   if ( ! $OPT_EXTENSIONS )
   {
-    die "--upgrade-path is only supported with --extensions"
+    die "--upgrade-path is only supported with --extension"
   }
   $OPT_UPGRADE = 1; # implied 
   my @path = split ('--', $OPT_UPGRADE_PATH);
   $OPT_UPGRADE_FROM = $path[0]
-    || die "Malformed upgrade path, <from>:<to> expected, $OPT_UPGRADE_PATH given";
+    || die "Malformed upgrade path, <from>--<to> expected, $OPT_UPGRADE_PATH given";
   $OPT_UPGRADE_TO = $path[1]
-    || die "Malformed upgrade path, <from>:<to> expected, $OPT_UPGRADE_PATH given";
+    || die "Malformed upgrade path, <from>--<to> expected, $OPT_UPGRADE_PATH given";
   print "Upgrade path: ${OPT_UPGRADE_FROM} --> ${OPT_UPGRADE_TO}\n";
 }
 
+if ( $OPT_EXTENSIONS )
+{
+	$OPT_WITH_RASTER = 1; # implied
+}
+
 
 
 ##################################################################
@@ -232,17 +250,84 @@ if ( ! $libver )
 }
 
 
+sub create_upgrade_test_objects
+{
+  # TODO: allow passing the "upgrade-init" script via commandline
+
+  my $query = "create table upgrade_test(g1 geometry, g2 geography";
+  $query .= ", r raster" if ( $OPT_WITH_RASTER );
+  $query .= ")";
+  my $ret = sql($query);
+  unless ( $ret =~ /^CREATE/ ) {
+    `dropdb $DB`;
+    print "\nSomething went wrong creating upgrade_test table: $ret.\n";
+    exit(1);
+  }
+
+  if ( $OPT_WITH_RASTER )
+  {
+    $query = "insert into upgrade_test(r) ";
+    $query .= "select ST_AddBand(ST_MakeEmptyRaster(10, 10, 1, 1, 2, 2, 0, 0,4326), 1, '8BSI'::text, -129, NULL);";
+    $query .= "set client_min_messages to error; select AddRasterConstraints('upgrade_test', 'r')";
+    $ret = sql($query);
+    unless ( $ret =~ /^t$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong adding raster constraints to upgrade_test: " . $ret . "\n";
+      exit(1);
+    }
+  }
+
+  if ( $OPT_WITH_TOPO )
+  {
+    $query = "select topology.createTopology('upgrade_test');";
+    $ret = sql($query);
+    unless ( $ret =~ /^[1-9][0-9]*$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong adding upgrade_test topology: " . $ret . "\n";
+      exit(1);
+    }
+  }
+}
+
+sub drop_upgrade_test_objects
+{
+  # TODO: allow passing the "upgrade-cleanup" script via commandline
+
+  my $ret = sql("drop table upgrade_test;");
+  unless ( $ret =~ /^DROP/ ) {
+    `dropdb $DB`;
+    print "\nSomething went wrong dropping spatial tables: $ret.\n";
+    exit(1);
+  }
+
+  if ( $OPT_WITH_TOPO )
+  {
+    my $query = "SELECT topology.DropTopology('upgrade_test');";
+    $ret = sql($query);
+    unless ( $ret =~ /^Topology 'upgrade_test' dropped$/ ) {
+      `dropdb $DB`;
+      print "\nSomething went wrong dropping upgrade_test topology: " . $ret . "\n";
+      exit(1);
+    }
+  }
+}
+
+
 if ( $OPT_UPGRADE )
 {
-	if ( $OPT_EXTENSIONS )
-	{	
-		upgrade_spatial_extensions();
-	}
-	else
-	{
+  create_upgrade_test_objects();
+
+  if ( $OPT_EXTENSIONS )
+  {
+    upgrade_spatial_extensions();
+  }
+  else
+  {
 	  upgrade_spatial();
   }
 
+  drop_upgrade_test_objects();
+
   # Update libver
   $libver = sql("select postgis_lib_version()");
 }
@@ -530,7 +615,7 @@ sub sql
 {
 	my $sql = shift;
 	my $result = `psql -tXA -d $DB -c "$sql"`;
-	$result =~ s/\n$//;
+	$result =~ s/[\n\r]*$//;
 	$result;
 }
 
@@ -598,6 +683,7 @@ sub run_simple_test
 	my $cmd = "psql -v \"VERBOSITY=terse\""
           . " -v \"tmpfile='$tmpfile'\""
           . " -v \"scriptdir=$scriptdir\""
+          . " -v \"regdir=$REGDIR\""
           . " -tXA $DB < $sql > $outfile 2>&1";
 	my $rv = system($cmd);
 
@@ -608,7 +694,7 @@ sub run_simple_test
 
 	# Strip the lines we don't care about
 	@lines = grep(!/^\$/, @lines);
-	@lines = grep(!/^(INSERT|DELETE|UPDATE|SELECT|COPY)/, @lines);
+	@lines = grep(!/^(INSERT|DELETE|UPDATE|SELECT|COPY|DO)/, @lines);
 	@lines = grep(!/^(CONTEXT|RESET|ANALYZE)/, @lines);
 	@lines = grep(!/^(DROP|CREATE|ALTER|VACUUM)/, @lines);
 	@lines = grep(!/^(LOG|SET|TRUNCATE)/, @lines);
@@ -624,6 +710,7 @@ sub run_simple_test
 		$lines[$i] =~ s/[eE]([+-])0+(\d+)/e$1$2/g;
 		$lines[$i] =~ s/Self-intersection .*/Self-intersection/;
 		$lines[$i] =~ s/^ROLLBACK/COMMIT/;
+		$lines[$i] =~ s/^psql.*(NOTICE|WARNING|ERROR):/\1:/g;
 	}
 	
 	# Write out output file
@@ -1143,8 +1230,11 @@ sub load_sql_file
 		my $cmd = "psql $psql_opts -Xf $file $DB >> $REGRESS_LOG 2>&1";
 		print "  $file\n" if $VERBOSE;
 		my $rv = system($cmd);
-		die "\nError encountered loading $file, see $REGRESS_LOG for details\n\n"
-		    if $rv;
+		if ( $rv )
+		{
+		  fail "Error encountered loading $file", $REGRESS_LOG;
+		  exit 1
+		}
 	}
 	return 1;
 }
@@ -1156,6 +1246,10 @@ sub prepare_spatial_extensions
 	my $psql_opts = "--no-psqlrc --variable ON_ERROR_STOP=true";
 	my $sql = "CREATE EXTENSION postgis";
 	if ( $OPT_UPGRADE_FROM ) {
+		if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+			prepare_spatial();
+			return;
+		}
 		$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
 	}
 
@@ -1182,6 +1276,21 @@ sub prepare_spatial_extensions
   		die;
 		}
  	}
+
+	if ( $OPT_WITH_SFCGAL )
+	{
+		my $sql = "CREATE EXTENSION postgis_sfcgal";
+		if ( $OPT_UPGRADE_FROM ) {
+			$sql .= " VERSION '" . $OPT_UPGRADE_FROM . "'";
+		}
+ 		$cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
+		$rv = system($cmd);
+		if ( $rv ) {
+		  fail "Error encountered creating EXTENSION POSTGIS_SFCGAL", $REGRESS_LOG;
+		  die;
+		}
+	}
+
  	return 1;
 }
 
@@ -1223,7 +1332,7 @@ sub upgrade_spatial
 {
     print "Upgrading PostGIS in '${DB}' \n" ;
 
-    my $script = `ls ${STAGED_SCRIPTS_DIR}/postgis_upgrade_*_minor.sql`;
+    my $script = `ls ${STAGED_SCRIPTS_DIR}/postgis_upgrade.sql`;
     chomp($script);
 
     if ( -e $script )
@@ -1238,7 +1347,7 @@ sub upgrade_spatial
     
     if ( $OPT_WITH_TOPO ) 
     {
-        my $script = `ls ${STAGED_SCRIPTS_DIR}/topology_upgrade_*_minor.sql`;
+        my $script = `ls ${STAGED_SCRIPTS_DIR}/topology_upgrade.sql`;
         chomp($script);
         if ( -e $script )
         {
@@ -1253,7 +1362,7 @@ sub upgrade_spatial
     
     if ( $OPT_WITH_RASTER ) 
     {
-        my $script = `ls ${STAGED_SCRIPTS_DIR}/rtpostgis_upgrade_*_minor.sql`;
+        my $script = `ls ${STAGED_SCRIPTS_DIR}/rtpostgis_upgrade.sql`;
         chomp($script);
         if ( -e $script )
         {
@@ -1276,6 +1385,10 @@ sub upgrade_spatial_extensions
     my $nextver = $OPT_UPGRADE_TO ? "${OPT_UPGRADE_TO}" : "${libver}next";
     my $sql = "ALTER EXTENSION postgis UPDATE TO '${nextver}'";
 
+    if ( $OPT_UPGRADE_FROM eq "unpackaged" ) {
+      $sql = "CREATE EXTENSION postgis VERSION '${nextver}' FROM unpackaged";
+    }
+
     print "Upgrading PostGIS in '${DB}' using: ${sql}\n" ;
 
     my $cmd = "psql $psql_opts -c \"" . $sql . "\" $DB >> $REGRESS_LOG 2>&1";
@@ -1330,10 +1443,20 @@ sub drop_spatial_extensions
     
     if ( $OPT_WITH_TOPO )
     {
-        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_topology\" $DB >> $REGRESS_LOG 2>&1";
+        # NOTE: "manually" dropping topology schema as EXTENSION does not
+        #       take care of that itself, see
+        #       http://trac.osgeo.org/postgis/ticket/2138
+        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_topology; DROP SCHEMA topology;\" $DB >> $REGRESS_LOG 2>&1";
         $rv = system($cmd);
       	$ok = 0 if $rv;
     }
+
+    if ( $OPT_WITH_SFCGAL )
+    {
+        $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis_sfcgal;\" $DB >> $REGRESS_LOG 2>&1";
+        $rv = system($cmd);
+        $ok = 0 if $rv;
+    }
     
     $cmd = "psql $psql_opts -c \"DROP EXTENSION postgis\" $DB >> $REGRESS_LOG 2>&1";
     $rv = system($cmd);
